@@ -19,5 +19,5 @@ let rec of_odoc ~cache otyp =
   | Arrow (_lbl, left, right) -> cache (Arrow (of_odoc ~cache left, of_odoc ~cache right))
   | Constr (name, args) ->
       cache (Constr (Typename.to_string name, List.map (of_odoc ~cache) args))
-  | Tuple li -> cache (Tuple (List.map (of_odoc ~cache) li))
+  | Tuple li -> cache (Tuple (List.map (fun (_, ty) -> of_odoc ~cache ty) li))
   | _ -> Unhandled
