@@ -15,21 +15,17 @@
  *)
 
 open Odoc_model
-open Odoc_model.Paths
 
 val compilation_unit : Lang.Compilation_unit.t -> Types.Document.t
 
 val page : Lang.Page.t -> Types.Document.t
 (** Convert compilation unit or page models into a document *)
 
-val source_tree : Lang.SourceTree.t -> Types.Document.t list
-
-val source_page :
-  Identifier.SourcePage.t ->
+val implementation :
+  Lang.Implementation.t ->
   Syntax_highlighter.infos ->
-  Lang.Source_info.infos ->
   string ->
-  Types.Document.t
+  Types.Document.t list
 
 val type_expr : ?needs_parentheses:bool -> Lang.TypeExpr.t -> Codefmt.t
 
