@@ -5,12 +5,19 @@ module rec Class : sig
 end =
   Class
 
-and InternalLink : sig
-  type target = Resolved of Url.t | Unresolved
-
-  type t = { target : target; content : Inline.t; tooltip : string option }
+and Link : sig
+  type t = { target : Target.t; content : Inline.t; tooltip : string option }
 end =
-  InternalLink
+  Link
+
+and Target : sig
+  type internal = Resolved of Url.t | Unresolved
+
+  type href = string
+
+  type t = Internal of internal | External of href
+end =
+  Target
 
 and Raw_markup : sig
   type target = Odoc_model.Comment.raw_markup_target
@@ -36,8 +43,6 @@ end =
 and Inline : sig
   type entity = string
 
-  type href = string
-
   type t = one list
 
   and one = { attr : Class.t; desc : desc }
@@ -47,8 +52,7 @@ and Inline : sig
     | Entity of entity
     | Linebreak
     | Styled of style * t
-    | Link of href * t
-    | InternalLink of InternalLink.t
+    | Link of Link.t
     | Source of Source.t
     | Math of Math.t
     | Raw_markup of Raw_markup.t
@@ -90,6 +94,9 @@ and Block : sig
     | Verbatim of string
     | Raw_markup of Raw_markup.t
     | Table of t Table.t
+    | Image of Target.t * string
+    | Video of Target.t * string
+    | Audio of Target.t * string
 
   and list_type = Ordered | Unordered
 end =
@@ -183,7 +190,11 @@ end =
   Page
 
 and Source_page : sig
-  type info = Syntax of string | Anchor of string | Link of Url.Anchor.t
+  type target = {
+    documentation : Url.Anchor.t option;
+    implementation : Url.Anchor.t option;
+  }
+  type info = Syntax of string | Anchor of string | Link of target
 
   type code = span list
   and span = Tagged_code of info * code | Plain_code of string
@@ -192,13 +203,8 @@ and Source_page : sig
 end =
   Source_page
 
-and Asset : sig
-  type t = { url : Url.Path.t; src : Fpath.t }
-end =
-  Asset
-
 module Document = struct
-  type t = Page of Page.t | Source_page of Source_page.t | Asset of Asset.t
+  type t = Page of Page.t | Source_page of Source_page.t
 end
 
 let inline ?(attr = []) desc = Inline.{ attr; desc }
