@@ -1,11 +1,22 @@
 val find_universe_and_version :
   string -> (string * string, [> `Msg of string ]) result
 
-val of_voodoo : string -> blessed:bool -> Packages.t list
+type pkg
+
+val find_pkg : string -> blessed:bool -> pkg option
+(** [get_pkg name ~blessed] looks for a package named [name] in the prep
+    directory *)
+
+val of_voodoo : pkg -> Packages.t
+
+val occurrence_file_of_pkg : pkg -> Fpath.t
+(** [occurrences_file_of_pkg pkg odoc_dir] returns an appropriate filename for
+    the occurrences file for [pkg]. *)
 
 type extra_paths = {
   pkgs : Fpath.t Util.StringMap.t;
   libs : Fpath.t Util.StringMap.t;
+  libs_of_pkg : string list Util.StringMap.t;
 }
 
 val empty_extra_paths : extra_paths
