@@ -12,6 +12,8 @@ val v :
   flat:bool ->
   open_details:bool ->
   as_json:bool ->
+  remap:(string * string) list ->
+  ?home_breadcrumb:string ->
   unit ->
   t
 (** [search_result] indicates whether this is a summary for a search result. In
@@ -34,3 +36,7 @@ val open_details : t -> bool
 val as_json : t -> bool
 
 val search_result : t -> bool
+
+val remap : t -> (string * string) list
+
+val home_breadcrumb : t -> string option
