@@ -13,11 +13,14 @@
  * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
  * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *)
+open Odoc_utils
 
+module HLink = Link
 open Odoc_document.Types
 module Html = Tyxml.Html
 module Doctree = Odoc_document.Doctree
 module Url = Odoc_document.Url
+module Link = HLink
 
 type any = Html_types.flow5
 
@@ -83,8 +86,8 @@ and source k ?a (t : Source.t) =
         let content = tokens l in
         if content = [] then [] else [ Html.span content ]
     | Tag (Some s, l) -> [ Html.span ~a:[ Html.a_class [ s ] ] (tokens l) ]
-  and tokens t = Utils.list_concat_map t ~f:token in
-  Utils.optional_elt Html.code ?a (tokens t)
+  and tokens t = List.concat_map token t in
+  match tokens t with [] -> [] | l -> [ Html.code ?a l ]
 
 and styled style ~emph_level =
   match style with
@@ -96,12 +99,12 @@ and styled style ~emph_level =
   | `Superscript -> (emph_level, Html.sup ~a:[])
   | `Subscript -> (emph_level, Html.sub ~a:[])
 
-let rec internallink ~config ~emph_level ~resolve ?(a = [])
-    { InternalLink.target; content; tooltip } =
+let rec internallink ~config ~emph_level ~resolve ?(a = []) target content
+    tooltip =
   let a = match tooltip with Some s -> Html.a_title s :: a | None -> a in
   let elt =
     match target with
-    | Resolved uri ->
+    | Target.Resolved uri ->
         let href = Link.href ~config ~resolve uri in
         let content = inline_nolink ~emph_level content in
         if Config.search_result config then
@@ -137,21 +140,22 @@ and inline ~config ?(emph_level = 0) ~resolve (l : Inline.t) :
     | Styled (style, c) ->
         let emph_level, app_style = styled style ~emph_level in
         [ app_style @@ inline ~config ~emph_level ~resolve c ]
-    | Link (_, c) when Config.search_result config ->
+    | Link { content = c; _ } when Config.search_result config ->
         (* When displaying for a search result, links are displayed as regular
            text. *)
         let content = inline_nolink ~emph_level c in
         [ Html.span ~a content ]
-    | Link (href, c) ->
+    | Link { target = External href; content = c; _ } ->
         let a = (a :> Html_types.a_attrib Html.attrib list) in
         let content = inline_nolink ~emph_level c in
         [ Html.a ~a:(Html.a_href href :: a) content ]
-    | InternalLink c -> internallink ~config ~emph_level ~resolve ~a c
+    | Link { target = Internal t; content; tooltip } ->
+        internallink ~config ~emph_level ~resolve ~a t content tooltip
     | Source c -> source (inline ~config ~emph_level ~resolve) ~a c
     | Math s -> [ inline_math s ]
     | Raw_markup r -> raw_markup r
   in
-  Utils.list_concat_map ~f:one l
+  List.concat_map one l
 
 and inline_nolink ?(emph_level = 0) (l : Inline.t) :
     non_link_phrasing Html.elt list =
@@ -168,12 +172,11 @@ and inline_nolink ?(emph_level = 0) (l : Inline.t) :
         let emph_level, app_style = styled style ~emph_level in
         [ app_style @@ inline_nolink ~emph_level c ]
     | Link _ -> assert false
-    | InternalLink _ -> assert false
     | Source c -> source (inline_nolink ~emph_level) ~a c
     | Math s -> [ inline_math s ]
     | Raw_markup r -> raw_markup r
   in
-  Utils.list_concat_map ~f:one l
+  List.concat_map one l
 
 let heading ~config ~resolve (h : Heading.t) =
   let a, anchor =
@@ -213,6 +216,20 @@ let rec block ~config ~resolve (l : Block.t) : flow Html.elt list =
       let a = Some (class_ (extra_class @ t.attr)) in
       [ mk ?a content ]
     in
+    let mk_media_block media_block target alt =
+      let block =
+        match target with
+        | Target.External url -> media_block url alt
+        | Internal (Resolved uri) ->
+            let url = Link.href ~config ~resolve uri in
+            media_block url alt
+        | Internal Unresolved ->
+            let content = [ Html.txt alt ] in
+            let a = Html.a_class [ "xref-unresolved" ] :: [] in
+            [ Html.span ~a content ]
+      in
+      mk_block Html.div block
+    in
     match t.desc with
     | Inline i ->
         if t.attr = [] then as_flow @@ inline ~config ~resolve i
@@ -243,8 +260,37 @@ let rec block ~config ~resolve (l : Block.t) : flow Html.elt list =
         let extra_class = [ "language-" ^ lang_tag ] in
         mk_block ~extra_class Html.pre (source (inline ~config ~resolve) c)
     | Math s -> mk_block Html.div [ block_math s ]
+    | Audio (target, alt) ->
+        let audio src alt =
+          [
+            Html.audio ~src
+              ~a:[ Html.a_controls (); Html.a_aria "label" [ alt ] ]
+              [];
+          ]
+        in
+        mk_media_block audio target alt
+    | Video (target, alt) ->
+        let video src alt =
+          [
+            Html.video ~src
+              ~a:[ Html.a_controls (); Html.a_aria "label" [ alt ] ]
+              [];
+          ]
+        in
+        mk_media_block video target alt
+    | Image (target, alt) ->
+        let image src alt =
+          let img =
+            Html.a
+              ~a:[ Html.a_href src; Html.a_class [ "img-link" ] ]
+              [ Html.img ~src ~alt () ]
+          in
+          [ img ]
+        in
+        mk_media_block image target alt
   in
-  Utils.list_concat_map l ~f:one
+
+  List.concat_map one l
 
 and mk_rows ~config ~resolve { align; data } =
   let mk_row row =
@@ -443,7 +489,7 @@ module Toc = struct
       in
       let title_str =
         List.map (Format.asprintf "%a" (Tyxml.Html.pp_elt ())) text
-        |> String.concat ""
+        |> String.concat ~sep:""
       in
       let href = Link.href ~config ~resolve url in
       { title; title_str; href; children = List.map section children }
@@ -454,24 +500,138 @@ end
 module Breadcrumbs = struct
   open Types
 
-  let gen_breadcrumbs ~config ~url =
-    let rec get_parent_paths x =
-      match x with
-      | [] -> []
-      | x :: xs -> (
-          match Odoc_document.Url.Path.of_list (List.rev (x :: xs)) with
-          | Some x -> x :: get_parent_paths xs
-          | None -> get_parent_paths xs)
+  let page_parent (page : Url.Path.t) =
+    let page =
+      match page with
+      | { parent = Some parent; name = "index"; kind = `LeafPage } -> parent
+      | _ -> page
     in
-    let to_breadcrumb path =
-      let href =
-        Link.href ~config ~resolve:(Current url)
-          (Odoc_document.Url.from_path path)
+    match page with
+    | { parent = None; name = "index"; kind = `LeafPage } -> None
+    | { parent = Some parent; _ } -> Some parent
+    | { parent = None; _ } ->
+        Some { Url.Path.parent = None; name = "index"; kind = `LeafPage }
+
+  let home_breadcrumb ~home_name config ~current_path ~home_path =
+    let href =
+      Some
+        (Link.href ~config ~resolve:(Current current_path)
+           (Odoc_document.Url.from_path home_path))
+    in
+    { href; name = [ Html.txt home_name ]; kind = `LeafPage }
+
+  let gen_breadcrumbs_no_sidebar ~config ~url =
+    let url =
+      match url with
+      | { Url.Path.name = "index"; parent = Some parent; kind = `LeafPage } ->
+          parent
+      | _ -> url
+    in
+    match url with
+    | { Url.Path.name = "index"; parent = None; kind = `LeafPage } ->
+        let kind = `LeafPage in
+        let current = { href = None; name = [ Html.txt "" ]; kind } in
+        { parents = []; up_url = None; current }
+    | url -> (
+        (* This is the pre 3.0 way of computing the breadcrumbs *)
+        let rec get_parent_paths x =
+          match x with
+          | [] -> []
+          | x :: xs -> (
+              match Odoc_document.Url.Path.of_list (List.rev (x :: xs)) with
+              | Some x -> x :: get_parent_paths xs
+              | None -> get_parent_paths xs)
+        in
+        let to_breadcrumb path =
+          let href =
+            Some
+              (Link.href ~config ~resolve:(Current url)
+                 (Odoc_document.Url.from_path path))
+          in
+          { href; name = [ Html.txt path.name ]; kind = path.kind }
+        in
+        let parent_paths =
+          get_parent_paths (List.rev (Odoc_document.Url.Path.to_list url))
+          |> List.rev
+        in
+        match List.rev parent_paths with
+        | [] -> assert false
+        | current :: parents ->
+            let up_url =
+              match page_parent current with
+              | None -> None
+              | Some up ->
+                  Some
+                    (Link.href ~config ~resolve:(Current url)
+                       (Odoc_document.Url.from_path up))
+            in
+            let current = to_breadcrumb current in
+            let parents = List.map to_breadcrumb parents |> List.rev in
+            let home =
+              home_breadcrumb ~home_name:"Index" config ~current_path:url
+                ~home_path:
+                  { Url.Path.name = "index"; parent = None; kind = `LeafPage }
+            in
+            { current; parents = home :: parents; up_url })
+
+  let gen_breadcrumbs_with_sidebar ~config ~sidebar ~url:current_url =
+    let find_parent =
+      List.find_opt (function
+        | ({ node = { url = { page; anchor = ""; _ }; _ }; _ } :
+            Odoc_document.Sidebar.entry Tree.t)
+          when Url.Path.is_prefix page current_url ->
+            true
+        | _ -> false)
+    in
+    let rec extract acc (tree : Odoc_document.Sidebar.t) =
+      let parent =
+        match find_parent tree with
+        | Some { node = { url; valid_link; content; _ }; children } ->
+            let href =
+              if valid_link then
+                Some (Link.href ~config ~resolve:(Current current_url) url)
+              else None
+            in
+            let name = inline_nolink content in
+            let breadcrumb = { href; name; kind = url.page.kind } in
+            if url.page = current_url then Some (`Current breadcrumb)
+            else Some (`Parent (breadcrumb, children))
+        | _ -> None
       in
-      { href; name = path.name; kind = path.kind }
+      match parent with
+      | Some (`Parent (bc, children)) -> extract (bc :: acc) children
+      | Some (`Current current) ->
+          let up_url =
+            List.find_map (fun (b : Types.breadcrumb) -> b.href) acc
+          in
+          { Types.current; parents = List.rev acc; up_url }
+      | None ->
+          let kind = current_url.kind and name = current_url.name in
+          let current = { href = None; name = [ Html.txt name ]; kind } in
+          let up_url =
+            List.find_map (fun (b : Types.breadcrumb) -> b.href) acc
+          in
+          let parents = List.rev acc in
+          { Types.current; parents; up_url }
     in
-    get_parent_paths (List.rev (Odoc_document.Url.Path.to_list url))
-    |> List.rev |> List.map to_breadcrumb
+    let escape =
+      match (Config.home_breadcrumb config, find_parent sidebar) with
+      | Some home_name, Some { node; _ } -> (
+          match page_parent node.url.page with
+          | None -> []
+          | Some parent ->
+              [
+                home_breadcrumb ~home_name config ~current_path:current_url
+                  ~home_path:parent;
+              ])
+      | _ -> []
+    in
+    extract escape sidebar
+
+  let gen_breadcrumbs ~config ~sidebar ~url =
+    match sidebar with
+    | None -> gen_breadcrumbs_no_sidebar ~config ~url
+    | Some sidebar -> gen_breadcrumbs_with_sidebar ~config ~sidebar ~url
 end
 
 module Page = struct
@@ -482,21 +642,33 @@ module Page = struct
         | `Closed | `Open | `Default -> None
         | `Inline -> Some 0)
 
-  let rec include_ ~config { Subpage.content; _ } = page ~config content
+  let rec include_ ~config ~sidebar { Subpage.content; _ } =
+    page ~config ~sidebar content
 
-  and subpages ~config subpages = List.map (include_ ~config) subpages
+  and subpages ~config ~sidebar subpages =
+    List.map (include_ ~config ~sidebar) subpages
 
-  and page ~config p : Odoc_document.Renderer.page =
-    let { Page.preamble; items = i; url; source_anchor } =
+  and page ~config ~sidebar p : Odoc_document.Renderer.page =
+    let { Page.preamble = _; items = i; url; source_anchor } =
       Doctree.Labels.disambiguate_page ~enter_subpages:false p
     in
-    let subpages = subpages ~config @@ Doctree.Subpages.compute p in
+    let subpages = subpages ~config ~sidebar @@ Doctree.Subpages.compute p in
     let resolve = Link.Current url in
+    let breadcrumbs = Breadcrumbs.gen_breadcrumbs ~config ~sidebar ~url in
+    let sidebar =
+      match sidebar with
+      | None -> None
+      | Some sidebar ->
+          let sidebar = Odoc_document.Sidebar.to_block sidebar url in
+          (Some (block ~config ~resolve sidebar) :> any Html.elt list option)
+    in
     let i = Doctree.Shift.compute ~on_sub i in
     let uses_katex = Doctree.Math.has_math_elements p in
     let toc = Toc.gen_toc ~config ~resolve ~path:url i in
-    let breadcrumbs = Breadcrumbs.gen_breadcrumbs ~config ~url in
     let content = (items ~config ~resolve i :> any Html.elt list) in
+    let header, preamble = Doctree.PageTitle.render_title ?source_anchor p in
+    let header = items ~config ~resolve header in
+    let preamble = items ~config ~resolve preamble in
     if Config.as_json config then
       let source_anchor =
         match source_anchor with
@@ -504,55 +676,47 @@ module Page = struct
         | None -> None
       in
       Html_fragment_json.make ~config
-        ~preamble:(items ~config ~resolve preamble :> any Html.elt list)
-        ~breadcrumbs ~toc ~url ~uses_katex ~source_anchor content subpages
-    else
-      let header =
-        items ~config ~resolve
-          (Doctree.PageTitle.render_title ?source_anchor p @ preamble)
-      in
-      Html_page.make ~config ~header ~toc ~breadcrumbs ~url ~uses_katex content
+        ~preamble:(preamble :> any Html.elt list)
+        ~header ~breadcrumbs ~toc ~url ~uses_katex ~source_anchor content
         subpages
+    else
+      Html_page.make ~sidebar ~config ~header:(header @ preamble) ~toc
+        ~breadcrumbs ~url ~uses_katex content subpages
 
-  and source_page ~config sp =
+  and source_page ~config ~sidebar sp =
     let { Source_page.url; contents } = sp in
     let resolve = Link.Current sp.url in
+    let breadcrumbs = Breadcrumbs.gen_breadcrumbs ~config ~sidebar ~url in
+    let sidebar =
+      match sidebar with
+      | None -> None
+      | Some sidebar ->
+          let sidebar = Odoc_document.Sidebar.to_block sidebar url in
+          (Some (block ~config ~resolve sidebar) :> any Html.elt list option)
+    in
     let title = url.Url.Path.name
     and doc = Html_source.html_of_doc ~config ~resolve contents in
-    let breadcrumbs = Breadcrumbs.gen_breadcrumbs ~config ~url in
     let header =
       items ~config ~resolve (Doctree.PageTitle.render_src_title sp)
     in
     if Config.as_json config then
-      Html_fragment_json.make_src ~config ~url ~breadcrumbs [ doc ]
-    else Html_page.make_src ~breadcrumbs ~header ~config ~url title [ doc ]
-
-  let asset ~config { Asset.url; src } =
-    let filename = Link.Path.as_filename ~is_flat:(Config.flat config) url in
-    let content ppf =
-      let ic = open_in_bin (Fpath.to_string src) in
-      let len = 1024 in
-      let buf = Bytes.create len in
-      let rec loop () =
-        let read = input ic buf 0 len in
-        if read = len then (
-          Format.fprintf ppf "%s" (Bytes.to_string buf);
-          loop ())
-        else if len > 0 then
-          let buf = Bytes.sub buf 0 read in
-          Format.fprintf ppf "%s" (Bytes.to_string buf)
-      in
-      loop ();
-      close_in ic
-    in
-    { Odoc_document.Renderer.filename; content; children = [] }
+      Html_fragment_json.make_src ~config ~url ~breadcrumbs ~sidebar ~header
+        [ doc ]
+    else
+      Html_page.make_src ~breadcrumbs ~header ~config ~url ~sidebar title
+        [ doc ]
 end
 
-let render ~config = function
-  | Document.Page page -> [ Page.page ~config page ]
-  | Source_page src -> [ Page.source_page ~config src ]
-  | Asset asset -> [ Page.asset ~config asset ]
+let render ~config ~sidebar = function
+  | Document.Page page -> [ Page.page ~config ~sidebar page ]
+  | Source_page src -> [ Page.source_page ~config ~sidebar src ]
+
+let filepath ~config url = Link.Path.as_filename ~config url
 
 let doc ~config ~xref_base_uri b =
   let resolve = Link.Base xref_base_uri in
   block ~config ~resolve b
+
+let inline ~config ~xref_base_uri b =
+  let resolve = Link.Base xref_base_uri in
+  inline ~config ~resolve b
