@@ -1,6 +1,12 @@
 #if OCAML_VERSION >= (4, 14, 0)
 
-(* open Odoc_model.Lang.Source_info *)
+let rec is_persistent : Path.t -> bool = function
+  | Path.Pident id -> Ident.persistent id
+  | Path.Pdot(p, _) -> is_persistent p
+  | Path.Papply(p, _) -> is_persistent p
+#if OCAML_VERSION >= (5,1,0)
+  | Path.Pextra_ty (p, _) -> is_persistent p
+#endif
 
 let pos_of_loc loc = (loc.Location.loc_start.pos_cnum, loc.loc_end.pos_cnum)
 
@@ -53,18 +59,12 @@ module Env = struct
     match item.str_desc with
     | Tstr_module mb -> module_binding env parent mb
     | Tstr_recmodule mbs -> module_bindings env parent mbs
-    | Tstr_modtype mtd -> module_type_decl env parent mtd
+    | Tstr_modtype mtd -> module_type_declaration env parent mtd
     | Tstr_open _ | Tstr_value _ | Tstr_class _ | Tstr_eval _
     | Tstr_class_type _ | Tstr_include _ | Tstr_attribute _ | Tstr_primitive _
     | Tstr_type _ | Tstr_typext _ | Tstr_exception _ ->
         ()
 
-  and module_type_decl env _parent mtd =
-    let id = Ident_env.find_module_type env mtd.mtd_id in
-    match mtd.mtd_type with
-    | None -> ()
-    | Some mty -> module_type env (id :> Identifier.Signature.t) mty
-
   and module_type env (parent : Identifier.Signature.t) mty =
     match mty.mty_desc with
     | Tmty_signature sg -> signature env (parent : Identifier.Signature.t) sg
@@ -144,14 +144,21 @@ module IdentHashtbl = Hashtbl.Make (struct
   let hash = Hashtbl.hash
 end)
 
+module AnnotHashtbl = Hashtbl.Make (struct
+  type t =
+    Odoc_model.Lang.Source_info.annotation Odoc_model.Lang.Source_info.with_pos
+  let equal l1 l2 = l1 = l2
+  let hash = Hashtbl.hash
+end)
+
 module UidHashtbl = Shape.Uid.Tbl
 
 (* Adds the local definitions found in traverse infos to the [loc_to_id] and
    [ident_to_id] tables. *)
-let populate_local_defs source_id poses loc_to_id ident_to_id =
+let populate_local_defs source_id poses loc_to_id local_ident_to_loc =
   List.iter
     (function
-      | Typedtree_traverse.Analysis.Definition id, loc ->
+      | Typedtree_traverse.Analysis.LocalDefinition id, loc ->
           let name =
             Odoc_model.Names.LocalName.make_std
               (Printf.sprintf "local_%s_%d" (Ident.name id) (counter ()))
@@ -159,8 +166,8 @@ let populate_local_defs source_id poses loc_to_id ident_to_id =
           let identifier =
             Odoc_model.Paths.Identifier.Mk.source_location_int (source_id, name)
           in
-          IdentHashtbl.add ident_to_id id identifier;
-          LocHashtbl.add loc_to_id loc identifier
+          LocHashtbl.add loc_to_id loc identifier;
+          IdentHashtbl.add local_ident_to_loc id loc
       | _ -> ())
     poses
 
@@ -191,7 +198,6 @@ let anchor_of_identifier id =
     | `Module (parent, name) ->
         let anchor = anchor `Module (ModuleName.to_string name) in
         continue anchor parent
-    | `SourceDir _ -> assert false
     | `ModuleType (parent, name) ->
         let anchor = anchor `ModuleType (ModuleTypeName.to_string name) in
         continue anchor parent
@@ -213,20 +219,18 @@ let anchor_of_identifier id =
         let anchor = anchor `Exception (ExceptionName.to_string name) in
         continue anchor parent
     | `Class (parent, name) ->
-        let anchor = anchor `Class (ClassName.to_string name) in
+        let anchor = anchor `Class (TypeName.to_string name) in
         continue anchor parent
     | `Page _ -> assert false
     | `LeafPage _ -> assert false
-    | `CoreType _ -> assert false
     | `SourceLocation _ -> assert false
     | `ClassType (parent, name) ->
-        let anchor = anchor `ClassType (ClassTypeName.to_string name) in
+        let anchor = anchor `ClassType (TypeName.to_string name) in
         continue anchor parent
     | `SourcePage _ -> assert false
     | `Value (parent, name) ->
         let anchor = anchor `Val (ValueName.to_string name) in
         continue anchor parent
-    | `CoreException _ -> assert false
     | `Constructor (parent, name) ->
         let anchor = anchor `Constructor (ConstructorName.to_string name) in
         continue anchor parent
@@ -281,20 +285,50 @@ let populate_global_defs env source_id loc_to_id uid_to_loc uid_to_id =
 
 (* Extract [Typedtree_traverse] occurrence information and turn them into proper
    source infos *)
-let process_occurrences poses uid_to_id ident_to_id =
-  List.filter_map
+let process_occurrences env poses loc_to_id local_ident_to_loc =
+  let open Odoc_model.Lang.Source_info in
+  (* Ensure source infos are not repeated by putting them in a Set (a unit hashtbl) *)
+  let occ_tbl = AnnotHashtbl.create 100 in
+  let process p find_in_env =
+    match p with
+    | Path.Pident id when IdentHashtbl.mem local_ident_to_loc id -> (
+        match
+          LocHashtbl.find_opt loc_to_id
+            (IdentHashtbl.find local_ident_to_loc id)
+        with
+        | None -> None
+        | Some id ->
+            let documentation = None and implementation = Some (Resolved id) in
+            Some { documentation; implementation })
+    | p -> (
+        match find_in_env env p with
+        | path ->
+            let documentation = if is_persistent p then Some path else None
+            and implementation = Some (Unresolved path) in
+            Some { documentation; implementation }
+        | exception _ -> None)
+  in
+  List.iter
     (function
-      | Typedtree_traverse.Analysis.Value (LocalValue uniq), loc -> (
-          match IdentHashtbl.find_opt ident_to_id uniq with
-          | Some anchor ->
-              Some (Odoc_model.Lang.Source_info.Value anchor, pos_of_loc loc)
-          | None -> None)
-      | Value (DefJmp x), loc -> (
-          match UidHashtbl.find_opt uid_to_id x with
-          | Some id -> Some (Value id, pos_of_loc loc)
-          | None -> None)
-      | Definition _, _ -> None)
-    poses
+      | Typedtree_traverse.Analysis.Value p, loc ->
+          process p Ident_env.Path.read_value
+          |> Option.iter @@ fun l ->
+             AnnotHashtbl.replace occ_tbl (Value l, pos_of_loc loc) ()
+      | Module p, loc ->
+          process p Ident_env.Path.read_module
+          |> Option.iter @@ fun l ->
+             AnnotHashtbl.replace occ_tbl (Module l, pos_of_loc loc) ()
+      | ModuleType p, loc ->
+          process p Ident_env.Path.read_module_type
+          |> Option.iter @@ fun l ->
+             AnnotHashtbl.replace occ_tbl (ModuleType l, pos_of_loc loc) ()
+      | Type p, loc ->
+          process p Ident_env.Path.read_type
+          |> Option.iter @@ fun l ->
+             AnnotHashtbl.replace occ_tbl (Type l, pos_of_loc loc) ()
+      | LocalDefinition _, _ -> ())
+    poses;
+  AnnotHashtbl.fold (fun k () acc -> k :: acc) occ_tbl []
 
 (* Add definition source info from the [loc_to_id] table *)
 let add_definitions loc_to_id occurrences =
@@ -303,64 +337,69 @@ let add_definitions loc_to_id occurrences =
       (Odoc_model.Lang.Source_info.Definition id, pos_of_loc loc) :: acc)
     loc_to_id occurrences
 
-let read_cmt_infos source_id_opt id cmt_info =
-  match Odoc_model.Compat.shape_of_cmt_infos cmt_info with
-  | Some shape -> (
-#if OCAML_VERSION >= (5, 2, 0)
-      let loc_of_declaration =
-        let open Typedtree in
-        function
-        | Value v -> v.val_loc
-        | Value_binding vb -> vb.vb_pat.pat_loc
-        | Type t -> t.typ_loc
-        | Constructor c -> c.cd_loc
-        | Extension_constructor e -> e.ext_loc
-        | Label l -> l.ld_loc
-        | Module m -> m.md_loc
-        | Module_substitution ms -> ms.ms_loc
-        | Module_binding mb -> mb.mb_loc
-        | Module_type mt -> mt.mtd_loc
-        | Class cd -> cd.ci_id_name.loc
-        | Class_type ctd -> ctd.ci_id_name.loc
+let read_cmt_infos source_id shape_info impl digest root imports =
+  match shape_info with
+  | Some (shape, uid_to_loc) ->
+      let fake_root_id =
+        Odoc_model.Paths.Identifier.Mk.root
+          (None, Odoc_model.Names.ModuleName.make_std "fake_root")
       in
-      let uid_to_loc = Shape.Uid.Tbl.map cmt_info.cmt_uid_to_decl loc_of_declaration in
-#else
-      let uid_to_loc = cmt_info.cmt_uid_to_loc in
-#endif
-      match (source_id_opt, cmt_info.cmt_annots) with
-      | Some source_id, Implementation impl ->
-          let env = Env.of_structure id impl in
-          let traverse_infos =
-            Typedtree_traverse.of_cmt env uid_to_loc impl |> List.rev
-            (* Information are accumulated in a list. We need to have the
-               first info first in the list, to assign anchors with increasing
-               numbers, so that adding some content at the end of a file does
-               not modify the anchors for existing anchors. *)
-          in
-          let loc_to_id = LocHashtbl.create 10
-          and ident_to_id = IdentHashtbl.create 10
-          and uid_to_id = UidHashtbl.create 10 in
-          let () =
-            (* populate [loc_to_id], [ident_to_id] and [uid_to_id] *)
-            populate_local_defs source_id traverse_infos loc_to_id ident_to_id;
+      let env = Env.of_structure fake_root_id impl in
+      let traverse_infos =
+        Typedtree_traverse.of_cmt env impl |> List.rev
+        (* Information are accumulated in a list. We need to have the
+           first info first in the list, to assign anchors with increasing
+           numbers, so that adding some content at the end of a file does
+           not modify the anchors for existing anchors. *)
+      in
+      let loc_to_id = LocHashtbl.create 10
+      and local_ident_to_loc = IdentHashtbl.create 10
+      and uid_to_id = UidHashtbl.create 10 in
+      let () =
+        match source_id with
+        | None -> ()
+        | Some source_id ->
+            populate_local_defs source_id traverse_infos loc_to_id
+              local_ident_to_loc;
             populate_global_defs env source_id loc_to_id uid_to_loc uid_to_id
-          in
-          let source_infos =
-            process_occurrences traverse_infos uid_to_id ident_to_id
-            |> add_definitions loc_to_id
-          in
-          ( Some (shape, Shape.Uid.Tbl.to_map uid_to_id),
-            Some
-              {
-                Odoc_model.Lang.Source_info.id = source_id;
-                infos = source_infos;
-              } )
-      | _, _ -> (Some (shape, Odoc_model.Compat.empty_map), None))
-  | None -> (None, None)
+      in
+      let source_infos =
+        process_occurrences env traverse_infos loc_to_id local_ident_to_loc
+        |> add_definitions loc_to_id
+      in
+      let shape_info = Some (shape, Shape.Uid.Tbl.to_map uid_to_id) in
+      {
+        Odoc_model.Lang.Implementation.id = source_id;
+        source_info = source_infos;
+        digest;
+        root;
+        linked = false;
+        shape_info;
+        imports;
+      }
+  | None as shape_info ->
+      {
+        Odoc_model.Lang.Implementation.id = source_id;
+        source_info = [];
+        digest;
+        root;
+        linked = false;
+        shape_info;
+        imports;
+      }
+
 
 #else
 
-let read_cmt_infos _source_id_opt _id _cmt_info =
-  (None, None)
+let read_cmt_infos source_id shape_info _impl digest root imports =
+  {
+    Odoc_model.Lang.Implementation.id = source_id;
+    source_info = [];
+    digest;
+    root;
+    linked = false;
+    shape_info;
+    imports;
+  }
 
 #endif
