@@ -14,9 +14,9 @@ module Names = struct
 
   let typename = To_string TypeName.to_string
 
-  let classname = To_string ClassName.to_string
+  let classname = To_string TypeName.to_string
 
-  let classtypename = To_string ClassTypeName.to_string
+  let classtypename = To_string TypeName.to_string
 
   let constructorname = To_string ConstructorName.to_string
 
@@ -36,6 +36,8 @@ module Names = struct
 
   let pagename = To_string PageName.to_string
 
+  let assetname = To_string AssetName.to_string
+
   let parametername = To_string ModuleName.to_string
 
   let defname = To_string DefName.to_string
@@ -77,7 +79,10 @@ module General_paths = struct
                 ((parent :> id_t option), name),
                 Pair (Option identifier, Names.pagename) )
         | `AssetFile (parent, name) ->
-            C ("`AssetFile", ((parent :> id_t), name), Pair (identifier, string))
+            C
+              ( "`AssetFile",
+                ((parent :> id_t), name),
+                Pair (identifier, Names.assetname) )
         | `Root (parent, name) ->
             C
               ( "`Root",
@@ -114,7 +119,6 @@ module General_paths = struct
               ( "`Type",
                 ((parent :> id_t), name),
                 Pair (identifier, Names.typename) )
-        | `CoreType name -> C ("`CoreType", name, Names.typename)
         | `Constructor (parent, name) ->
             C
               ( "`Constructor",
@@ -140,7 +144,6 @@ module General_paths = struct
               ( "`Exception",
                 ((parent :> id_t), name),
                 Pair (identifier, Names.exceptionname) )
-        | `CoreException name -> C ("`CoreException", name, Names.exceptionname)
         | `Value (parent, name) ->
             C
               ( "`Value",
@@ -161,8 +164,6 @@ module General_paths = struct
               ( "`Label",
                 ((parent :> id_t), name),
                 Pair (identifier, Names.labelname) )
-        | `SourceDir (parent, name) ->
-            C ("`SourceDir", ((parent :> id_t), name), Pair (identifier, string))
         | `SourcePage (parent, name) ->
             C
               ( "`SourcePage",
@@ -197,6 +198,7 @@ module General_paths = struct
       | `TModule -> C0 "`TModule"
       | `TModuleType -> C0 "`TModuleType"
       | `TPage -> C0 "`TPage"
+      | `TAsset -> C0 "`TAsset"
       | `TType -> C0 "`TType"
       | `TUnknown -> C0 "`TUnknown"
       | `TValue -> C0 "`TValue"
@@ -209,16 +211,28 @@ module General_paths = struct
       | `Resolved x -> C ("`Resolved", x, resolved_path)
       | `Identifier (x1, x2) ->
           C ("`Identifier", ((x1 :> id_t), x2), Pair (identifier, bool))
-      | `Root x -> C ("`Root", x, string)
+      | `Root x -> C ("`Root", x, Names.modulename)
       | `Forward x -> C ("`Forward", x, string)
-      | `Dot (x1, x2) -> C ("`Dot", ((x1 :> p), x2), Pair (path, string))
+      | `Dot (x1, x2) ->
+          C ("`Dot", ((x1 :> p), x2), Pair (path, Names.modulename))
+      | `DotT (x1, x2) ->
+          C ("`DotT", ((x1 :> p), x2), Pair (path, Names.typename))
+      | `DotMT (x1, x2) ->
+          C ("`DotMT", ((x1 :> p), x2), Pair (path, Names.moduletypename))
+      | `DotV (x1, x2) ->
+          C ("`DotV", ((x1 :> p), x2), Pair (path, Names.valuename))
       | `Apply (x1, x2) ->
-          C ("`Apply", ((x1 :> p), (x2 :> p)), Pair (path, path)))
+          C ("`Apply", ((x1 :> p), (x2 :> p)), Pair (path, path))
+      | `Substituted m -> C ("`Substituted", (m :> p), path)
+      | `SubstitutedMT m -> C ("`SubstitutedMT", (m :> p), path)
+      | `SubstitutedT m -> C ("`SubstitutedT", (m :> p), path)
+      | `SubstitutedCT m -> C ("`SubstitutedCT", (m :> p), path))
 
   and resolved_path : rp t =
     Variant
       (function
       | `Identifier x -> C ("`Identifier", x, identifier)
+      | `CoreType n -> C ("`CoreType", n, Names.typename)
       | `Subst (x1, x2) ->
           C
             ( "`Subst",
@@ -262,28 +276,32 @@ module General_paths = struct
             ( "`CanonicalType",
               ((x1 :> rp), (x2 :> p)),
               Pair (resolved_path, path) )
-      | `CanonicalDataType (x1, x2) ->
-          C
-            ( "`CanonicalDataType",
-              ((x1 :> rp), (x2 :> p)),
-              Pair (resolved_path, path) )
       | `OpaqueModuleType x -> C ("`OpaqueModuleType", (x :> rp), resolved_path)
       | `Type (x1, x2) ->
           C ("`Type", ((x1 :> rp), x2), Pair (resolved_path, Names.typename))
       | `Value (x1, x2) ->
           C ("`Value", ((x1 :> rp), x2), Pair (resolved_path, Names.valuename))
-      | `Constructor (x1, x2) ->
-          C
-            ( "`Constructor",
-              ((x1 :> rp), x2),
-              Pair (resolved_path, Names.constructorname) )
       | `Class (x1, x2) ->
           C ("`Class", ((x1 :> rp), x2), Pair (resolved_path, Names.classname))
       | `ClassType (x1, x2) ->
           C
             ( "`ClassType",
               ((x1 :> rp), x2),
-              Pair (resolved_path, Names.classtypename) ))
+              Pair (resolved_path, Names.classtypename) )
+      | `Substituted c -> C ("`Substituted", (c :> rp), resolved_path)
+      | `SubstitutedMT c -> C ("`SubstitutedMT", (c :> rp), resolved_path)
+      | `SubstitutedT c -> C ("`SubstitutedT", (c :> rp), resolved_path)
+      | `SubstitutedCT c -> C ("`SubstitutedCT", (c :> rp), resolved_path))
+
+  and hierarchy_reference : Paths.Reference.Hierarchy.t t =
+    let tag_page_path =
+      Variant
+        (function
+        | `TRelativePath -> C0 "`TRelativePath"
+        | `TAbsolutePath -> C0 "`TAbsolutePath"
+        | `TCurrentPackage -> C0 "`TCurrentPackage")
+    in
+    Pair (tag_page_path, List string)
 
   and reference : r t =
     Variant
@@ -291,6 +309,10 @@ module General_paths = struct
       | `Resolved x -> C ("`Resolved", x, resolved_reference)
       | `Root (x1, x2) -> C ("`Root", (x1, x2), Pair (string, reference_tag))
       | `Dot (x1, x2) -> C ("`Dot", ((x1 :> r), x2), Pair (reference, string))
+      | `Page_path x -> C ("`Page_path", x, hierarchy_reference)
+      | `Asset_path x -> C ("`Asset_path", x, hierarchy_reference)
+      | `Module_path x -> C ("`Module_path", x, hierarchy_reference)
+      | `Any_path x -> C ("`Any_path", x, hierarchy_reference)
       | `Module (x1, x2) ->
           C ("`Module", ((x1 :> r), x2), Pair (reference, Names.modulename))
       | `ModuleType (x1, x2) ->
@@ -359,6 +381,11 @@ module General_paths = struct
             ( "`Constructor",
               ((x1 :> rr), x2),
               Pair (resolved_reference, Names.constructorname) )
+      | `PolyConstructor (x1, x2) ->
+          C
+            ( "`PolyConstructor",
+              ((x1 :> rr), x2),
+              Pair (resolved_reference, Names.constructorname) )
       | `Exception (x1, x2) ->
           C
             ( "`Exception",
