@@ -4,38 +4,41 @@
    output the result to. *)
 
 open Odoc_utils
+open ResultMonad
 module List = ListLabels
 open Odoc_odoc
-open Compatcmdliner
+open Cmdliner
 
 let convert_syntax : Odoc_document.Renderer.syntax Arg.conv =
   let syntax_parser str =
     match str with
-    | "ml" | "ocaml" -> `Ok Odoc_document.Renderer.OCaml
-    | "re" | "reason" -> `Ok Odoc_document.Renderer.Reason
-    | s -> `Error (Printf.sprintf "Unknown syntax '%s'" s)
+    | "ml" | "ocaml" -> Ok Odoc_document.Renderer.OCaml
+    | "re" | "reason" -> Ok Odoc_document.Renderer.Reason
+    | s -> Error (Printf.sprintf "Unknown syntax '%s'" s)
   in
   let syntax_printer fmt syntax =
     Format.pp_print_string fmt (Odoc_document.Renderer.string_of_syntax syntax)
   in
-  (syntax_parser, syntax_printer)
+  Arg.conv' (syntax_parser, syntax_printer)
 
 let convert_directory ?(create = false) () : Fs.Directory.t Arg.conv =
-  let dir_parser, dir_printer = Arg.string in
+  let dir_parser, dir_printer =
+    (Arg.conv_parser Arg.string, Arg.conv_printer Arg.string)
+  in
   let odoc_dir_parser str =
     let () = if create then Fs.Directory.(mkdir_p (of_string str)) in
     match dir_parser str with
-    | `Ok res -> `Ok (Fs.Directory.of_string res)
-    | `Error e -> `Error e
+    | Ok res -> Ok (Fs.Directory.of_string res)
+    | Error (`Msg e) -> Error e
   in
   let odoc_dir_printer fmt dir = dir_printer fmt (Fs.Directory.to_string dir) in
-  (odoc_dir_parser, odoc_dir_printer)
+  Arg.conv' (odoc_dir_parser, odoc_dir_printer)
 
 (** On top of the conversion 'file' that checks that the passed file exists. *)
 let convert_fpath =
   let parse inp =
     match Arg.(conv_parser file) inp with
-    | Ok s -> Result.Ok (Fs.File.of_string s)
+    | Ok s -> Ok (Fs.File.of_string s)
     | Error _ as e -> e
   and print = Fpath.pp in
   Arg.conv (parse, print)
@@ -43,7 +46,7 @@ let convert_fpath =
 let convert_named_root =
   let parse inp =
     match String.cuts inp ~sep:":" with
-    | [ s1; s2 ] -> Result.Ok (s1, Fs.Directory.of_string s2)
+    | [ s1; s2 ] -> Ok (s1, Fs.Directory.of_string s2)
     | _ -> Error (`Msg "")
   in
   let print ppf (s, t) =
@@ -52,7 +55,7 @@ let convert_named_root =
   Arg.conv (parse, print)
 
 let handle_error = function
-  | Result.Ok () -> ()
+  | Ok () -> ()
   | Error (`Cli_error msg) ->
       Printf.eprintf "%s\n%!" msg;
       exit 2
@@ -84,7 +87,7 @@ module Antichain = struct
             rest
           && check rest
     in
-    if check l then Result.Ok ()
+    if check l then Ok ()
     else
       let msg =
         Format.sprintf "Paths given to all %s options must be disjoint" opt
@@ -124,7 +127,7 @@ let warnings_options =
   let warn_error =
     let doc = "Turn warnings into errors." in
     let env =
-      Arg.env_var "ODOC_WARN_ERROR" ~doc:(doc ^ " See option $(opt).")
+      Cmd.Env.info "ODOC_WARN_ERROR" ~doc:(doc ^ " See option $(opt).")
     in
     Arg.(value & flag & info ~docs ~doc ~env [ "warn-error" ])
   in
@@ -133,7 +136,7 @@ let warnings_options =
       "Whether warnings should be printed to stderr. See the $(b,errors) \
        command."
     in
-    let env = Arg.env_var "ODOC_PRINT_WARNINGS" ~doc in
+    let env = Cmd.Env.info "ODOC_PRINT_WARNINGS" ~doc in
     Arg.(value & opt bool true & info ~docs ~doc ~env [ "print-warnings" ])
   in
   let enable_missing_root_warning =
@@ -141,7 +144,7 @@ let warnings_options =
       "Produce a warning when a root is missing. This is usually a build \
        system problem so is disabled for users by default."
     in
-    let env = Arg.env_var "ODOC_ENABLE_MISSING_ROOT_WARNING" ~doc in
+    let env = Cmd.Env.info "ODOC_ENABLE_MISSING_ROOT_WARNING" ~doc in
     Arg.(value & flag & info ~docs ~doc ~env [ "enable-missing-root-warning" ])
   in
   let warnings_tag =
@@ -153,7 +156,7 @@ let warnings_options =
        command is passed the tag via the --warnings-tags parameter. A suitable \
        tag would be the name of the package."
     in
-    let env = Arg.env_var "ODOC_WARNINGS_TAG" ~doc in
+    let env = Cmd.Env.info "ODOC_WARNINGS_TAG" ~doc in
     Arg.(
       value & opt (some string) None & info ~docs ~doc ~env [ "warnings-tag" ])
   in
@@ -188,7 +191,7 @@ module Compile : sig
 
   val cmd : unit Term.t
 
-  val info : docs:string -> Term.info
+  val info : docs:string -> Cmd.info
 end = struct
   let has_page_prefix file =
     file |> Fs.File.basename |> Fs.File.to_string
@@ -223,7 +226,6 @@ end = struct
   let compile hidden directories resolve_fwd_refs dst output_dir package_opt
       parent_name_opt parent_id_opt open_modules children input warnings_options
       unique_id short_title =
-    let open Or_error in
     let _ =
       match unique_id with
       | Some id -> Odoc_model.Names.set_unique_ident id
@@ -354,7 +356,7 @@ end = struct
       "Compile a $(i,.cmti), $(i,.cmt), $(i,.cmi) or $(i,.mld) file to an \
        $(i,.odoc) file."
     in
-    Term.info "compile" ~docs ~doc ~man
+    Cmd.info "compile" ~docs ~doc ~man
 end
 
 module Compile_asset = struct
@@ -397,7 +399,7 @@ module Compile_asset = struct
       ]
     in
     let doc = "Declare the name of an asset." in
-    Term.info "compile-asset" ~docs ~doc ~man
+    Cmd.info "compile-asset" ~docs ~doc ~man
 end
 
 module Compile_impl = struct
@@ -470,12 +472,10 @@ module Compile_impl = struct
        containing the implementation information needed by odoc for the \
        compilation unit."
     in
-    Term.info "compile-impl" ~docs ~doc
+    Cmd.info "compile-impl" ~docs ~doc
 end
 
 module Indexing = struct
-  open Or_error
-
   let output_file ~dst marshall =
     match (dst, marshall) with
     | Some file, `JSON
@@ -572,12 +572,10 @@ module Indexing = struct
       "Generate an index of all identified entries in the .odocl files found \
        in the given directories."
     in
-    Term.info "compile-index" ~docs ~doc
+    Cmd.info "compile-index" ~docs ~doc
 end
 
 module Sidebar = struct
-  open Or_error
-
   let output_file ~dst marshall =
     match (dst, marshall) with
     | Some file, `JSON when not (Fpath.has_ext "json" (Fpath.v file)) ->
@@ -625,7 +623,7 @@ module Sidebar = struct
 
   let info ~docs =
     let doc = "Generate a sidebar from an index file." in
-    Term.info "sidebar-generate" ~docs ~doc
+    Cmd.info "sidebar-generate" ~docs ~doc
 end
 
 module Support_files_command = struct
@@ -643,7 +641,7 @@ module Support_files_command = struct
       "Copy the support files (e.g. default theme, JavaScript files) to the \
        output directory."
     in
-    Term.info ~docs ~doc "support-files"
+    Cmd.info ~docs ~doc "support-files"
 end
 
 module Css = struct
@@ -654,21 +652,19 @@ module Css = struct
       "DEPRECATED: Use $(i,odoc support-files) to copy the CSS file for the \
        default theme."
     in
-    Term.info ~docs ~doc "css"
+    Cmd.info ~docs ~doc "css"
 end
 
 module Odoc_link : sig
   val cmd : unit Term.t
 
-  val info : docs:string -> Term.info
+  val info : docs:string -> Cmd.info
 end = struct
   let get_output_file ~output_file ~input =
     match output_file with
     | Some file -> Fs.File.of_string file
     | None -> Fs.File.(set_ext ".odocl" input)
 
-  open Or_error
-
   (** Find the package/library name the output is part of *)
   let find_root_of_input l o =
     let l =
@@ -718,7 +714,7 @@ end = struct
       "Filter warnings that were compiled with a tag that is not in the list \
        of --warnings-tags passed."
     in
-    let env = Arg.env_var "ODOC_WARNINGS_TAGS" ~doc in
+    let env = Cmd.Env.info "ODOC_WARNINGS_TAGS" ~doc in
     Arg.(value & opt_all string [] & info ~docs ~doc ~env [ "warnings-tags" ])
 
   let link directories page_roots lib_roots input_file output_file
@@ -836,7 +832,7 @@ end = struct
     let doc =
       "Second stage of compilation. Link a $(i,.odoc) into a $(i,.odocl)."
     in
-    Term.info ~docs ~doc ~man "link"
+    Cmd.info ~docs ~doc ~man "link"
 end
 
 module type S = sig
@@ -848,17 +844,17 @@ module type S = sig
 end
 
 module Make_renderer (R : S) : sig
-  val process : docs:string -> unit Term.t * Term.info
+  val process : docs:string -> unit Term.t * Cmd.info
 
-  val targets : docs:string -> unit Term.t * Term.info
+  val targets : docs:string -> unit Term.t * Cmd.info
 
-  val targets_source : docs:string -> unit Term.t * Term.info
+  val targets_source : docs:string -> unit Term.t * Cmd.info
 
-  val generate : docs:string -> unit Term.t * Term.info
+  val generate : docs:string -> unit Term.t * Cmd.info
 
-  val generate_source : docs:string -> unit Term.t * Term.info
+  val generate_source : docs:string -> unit Term.t * Cmd.info
 
-  val generate_asset : docs:string -> unit Term.t * Term.info
+  val generate_asset : docs:string -> unit Term.t * Cmd.info
 end = struct
   let input_odoc =
     let doc = "Input file." in
@@ -882,10 +878,10 @@ end = struct
     let cmd =
       let syntax =
         let doc = "Available options: ml | re" in
-        let env = Arg.env_var "ODOC_SYNTAX" in
+        let env = Cmd.Env.info "ODOC_SYNTAX" in
         Arg.(
           value
-          & opt (pconv convert_syntax) Odoc_document.Renderer.OCaml
+          & opt convert_syntax Odoc_document.Renderer.OCaml
             @@ info ~docv:"SYNTAX" ~doc ~env [ "syntax" ])
       in
       Term.(
@@ -900,7 +896,7 @@ end = struct
            should be used instead."
           R.renderer.name R.renderer.name
       in
-      Term.info ~docs ~doc R.renderer.name
+      Cmd.info ~docs ~doc R.renderer.name
   end
 
   let process ~docs = Process.(cmd, info ~docs)
@@ -922,10 +918,10 @@ end = struct
     let cmd =
       let syntax =
         let doc = "Available options: ml | re" in
-        let env = Arg.env_var "ODOC_SYNTAX" in
+        let env = Cmd.Env.info "ODOC_SYNTAX" in
         Arg.(
           value
-          & opt (pconv convert_syntax) Odoc_document.Renderer.OCaml
+          & opt convert_syntax Odoc_document.Renderer.OCaml
             @@ info ~docv:"SYNTAX" ~doc ~env [ "syntax" ])
       in
       Term.(
@@ -937,7 +933,7 @@ end = struct
       let doc =
         Format.sprintf "Generate %s files from a $(i,.odocl)." R.renderer.name
       in
-      Term.info ~docs ~doc (R.renderer.name ^ "-generate")
+      Cmd.info ~docs ~doc (R.renderer.name ^ "-generate")
   end
 
   let generate ~docs = Generate.(cmd, info ~docs)
@@ -966,10 +962,10 @@ end = struct
     let cmd =
       let syntax =
         let doc = "Available options: ml | re" in
-        let env = Arg.env_var "ODOC_SYNTAX" in
+        let env = Cmd.Env.info "ODOC_SYNTAX" in
         Arg.(
           value
-          & opt (pconv convert_syntax) Odoc_document.Renderer.OCaml
+          & opt convert_syntax Odoc_document.Renderer.OCaml
             @@ info ~docv:"SYNTAX" ~doc ~env [ "syntax" ])
       in
       let sidebar = Generate.sidebar in
@@ -984,7 +980,7 @@ end = struct
         Format.sprintf "Generate %s files from a $(i,impl-*.odocl)."
           R.renderer.name
       in
-      Term.info ~docs ~doc (R.renderer.name ^ "-generate-source")
+      Cmd.info ~docs ~doc (R.renderer.name ^ "-generate-source")
   end
 
   let generate_source ~docs = Generate_source.(cmd, info ~docs)
@@ -1020,7 +1016,7 @@ end = struct
         Format.sprintf "Generate %s files from a $(i,impl-*.odocl)."
           R.renderer.name
       in
-      Term.info ~docs ~doc (R.renderer.name ^ "-generate-asset")
+      Cmd.info ~docs ~doc (R.renderer.name ^ "-generate-asset")
   end
 
   let generate_asset ~docs = Generate_asset.(cmd, info ~docs)
@@ -1064,7 +1060,7 @@ end = struct
           "Print the files that would be generated by $(i,%s-generate)."
           R.renderer.name
       in
-      Term.info (R.renderer.name ^ "-targets") ~docs ~doc
+      Cmd.info (R.renderer.name ^ "-targets") ~docs ~doc
   end
 
   let targets ~docs = Targets.(cmd, info ~docs)
@@ -1096,7 +1092,7 @@ end = struct
           "Print the files that would be generated by $(i,%s-generate-source)."
           R.renderer.name
       in
-      Term.info (R.renderer.name ^ "-targets-source") ~docs ~doc
+      Cmd.info (R.renderer.name ^ "-targets-source") ~docs ~doc
   end
 
   let targets_source ~docs = Targets_source.(cmd, info ~docs)
@@ -1105,7 +1101,7 @@ end
 module Odoc_latex_url : sig
   val cmd : unit Term.t
 
-  val info : docs:string -> Term.info
+  val info : docs:string -> Cmd.info
 end = struct
   let reference =
     let doc = "The reference to be resolved and whose url to be generated." in
@@ -1119,7 +1115,7 @@ end = struct
       $ (const reference_to_url $ odoc_file_directories $ reference))
 
   let info ~docs =
-    Term.info ~docs ~doc:"Resolve a reference and output its corresponding url."
+    Cmd.info ~docs ~doc:"Resolve a reference and output its corresponding url."
       "latex-url"
 end
 
@@ -1160,7 +1156,7 @@ module Odoc_html_args = struct
 
     let convert_dir : uri Arg.conv =
       let parser str =
-        if String.length str = 0 then `Error "invalid URI"
+        if String.length str = 0 then Error "invalid URI"
         else
           (* The URI is absolute if it starts with a scheme or with '/'. *)
           let last_char = str.[String.length str - 1] in
@@ -1168,7 +1164,7 @@ module Odoc_html_args = struct
             if last_char <> '/' then str
             else String.with_range ~len:(String.length str - 1) str
           in
-          `Ok
+          Ok
             (if is_absolute str then (Absolute str : uri)
              else
                Relative
@@ -1181,11 +1177,11 @@ module Odoc_html_args = struct
         | (Absolute uri : uri) -> Format.pp_print_string ppf uri
         | Relative _uri -> Format.pp_print_string ppf ""
       in
-      (parser, printer)
+      Arg.conv' (parser, printer)
 
     let convert_file_uri : Odoc_html.Types.file_uri Arg.conv =
       let parser str =
-        if String.length str = 0 then `Error "invalid URI"
+        if String.length str = 0 then Error "invalid URI"
         else
           let conv_rel_file rel =
             match String.cut ~rev:true ~sep:"/" rel with
@@ -1197,7 +1193,7 @@ module Odoc_html_args = struct
                 Odoc_document.Url.Path.
                   { kind = `File; parent = None; name = rel }
           in
-          `Ok
+          Ok
             (if is_absolute str then (Absolute str : file_uri)
              else Relative (conv_rel_file str))
       in
@@ -1205,7 +1201,7 @@ module Odoc_html_args = struct
         | Odoc_html.Types.Absolute uri -> Format.pp_print_string ppf uri
         | Odoc_html.Types.Relative _uri -> Format.pp_print_string ppf ""
       in
-      (parser, printer)
+      Arg.conv' (parser, printer)
   end
 
   let home_breadcrumb =
@@ -1271,7 +1267,7 @@ module Odoc_html_args = struct
     let convert_remap =
       let parse inp =
         match String.cut ~sep:":" inp with
-        | Some (orig, mapped) -> Result.Ok (orig, mapped)
+        | Some (orig, mapped) -> Ok (orig, mapped)
         | _ -> Error (`Msg "Map must be of the form '<orig>:https://...'")
       and print fmt (orig, mapped) = Format.fprintf fmt "%s:%s" orig mapped in
       Arg.conv (parse, print)
@@ -1312,10 +1308,22 @@ end
 
 module Odoc_html = Make_renderer (Odoc_html_args)
 
+module Odoc_markdown_cmd = Make_renderer (struct
+  type args = Odoc_markdown.Config.t
+
+  let render config _sidebar page = Odoc_markdown.Generator.render ~config page
+
+  let filepath config url = Odoc_markdown.Generator.filepath ~config url
+
+  let extra_args =
+    Term.const { Odoc_markdown.Config.root_url = None; allow_html = true }
+  let renderer = { Odoc_document.Renderer.name = "markdown"; render; filepath }
+end)
+
 module Odoc_html_url : sig
   val cmd : unit Term.t
 
-  val info : docs:string -> Term.info
+  val info : docs:string -> Cmd.info
 end = struct
   let root_url =
     let doc =
@@ -1337,14 +1345,14 @@ end = struct
        $ odoc_file_directories $ reference))
 
   let info ~docs =
-    Term.info ~docs ~doc:"Resolve a reference and output its corresponding url."
+    Cmd.info ~docs ~doc:"Resolve a reference and output its corresponding url."
       "html-url"
 end
 
 module Html_fragment : sig
   val cmd : unit Term.t
 
-  val info : docs:string -> Term.info
+  val info : docs:string -> Cmd.info
 end = struct
   let html_fragment directories xref_base_uri output_file input_file
       warnings_options =
@@ -1387,7 +1395,7 @@ end = struct
        $ input $ warnings_options))
 
   let info ~docs =
-    Term.info ~docs ~doc:"Generates an html fragment file from an mld one."
+    Cmd.info ~docs ~doc:"Generates an html fragment file from an mld one."
       "html-fragment"
 end
 
@@ -1408,9 +1416,25 @@ module Odoc_latex = Make_renderer (struct
     let doc = "Include children at the end of the page." in
     Arg.(value & opt bool true & info ~docv:"BOOL" ~doc [ "with-children" ])
 
+  let shorten_beyond_depth =
+    let doc = "Shorten items beyond the given depth." in
+    Arg.(
+      value
+      & opt (some' int) None
+      & info ~docv:"INT" ~doc [ "shorten-beyond-depth" ])
+
+  let remove_functor_arg_link =
+    let doc = "Remove link to functor argument." in
+    Arg.(
+      value & opt bool false
+      & info ~docv:"BOOL" ~doc [ "remove-functor-arg-link" ])
+
   let extra_args =
-    let f with_children = { Latex.with_children } in
-    Term.(const f $ with_children)
+    let f with_children shorten_beyond_depth remove_functor_arg_link =
+      { Latex.with_children; shorten_beyond_depth; remove_functor_arg_link }
+    in
+    Term.(
+      const f $ with_children $ shorten_beyond_depth $ remove_functor_arg_link)
 end)
 
 module Depends = struct
@@ -1448,7 +1472,7 @@ module Depends = struct
       Term.(const list_dependencies $ input)
 
     let info ~docs =
-      Term.info "compile-deps" ~docs
+      Cmd.info "compile-deps" ~docs
         ~doc:
           "List units (with their digest) which needs to be compiled in order \
            to compile this one. The unit itself and its digest is also \
@@ -1473,7 +1497,6 @@ module Depends = struct
       | Some p -> Format.fprintf pp "%a/" fmt_page p
 
     let list_dependencies input_file =
-      let open Or_error in
       Depends.for_rendering_step (Fs.Directory.of_string input_file)
       >>= fun depends ->
       List.iter depends ~f:(fun (root : Odoc_model.Root.t) ->
@@ -1496,7 +1519,7 @@ module Depends = struct
       Term.(const handle_error $ (const list_dependencies $ input))
 
     let info ~docs =
-      Term.info "link-deps" ~docs
+      Cmd.info "link-deps" ~docs
         ~doc:
           "Lists a subset of the packages and modules which need to be in \
            odoc's load path to link the $(i, odoc) files in the given \
@@ -1521,7 +1544,7 @@ module Depends = struct
       Term.(const handle_error $ (const cmd $ includes $ input))
 
     let info ~docs =
-      Term.info "html-deps" ~docs ~doc:"DEPRECATED: alias for link-deps"
+      Cmd.info "html-deps" ~docs ~doc:"DEPRECATED: alias for link-deps"
   end
 end
 
@@ -1536,7 +1559,7 @@ module Targets = struct
     let cmd = Term.(const list_targets $ Compile.dst $ Compile.input)
 
     let info ~docs =
-      Term.info "compile-targets" ~docs
+      Cmd.info "compile-targets" ~docs
         ~doc:
           "Print the name of the file produced by $(i,compile). If $(i,-o) is \
            passed, the same path is printed but error checking is performed."
@@ -1550,15 +1573,13 @@ module Targets = struct
       Term.(const list_targets $ Support_files_command.without_theme $ dst ())
 
     let info ~docs =
-      Term.info "support-files-targets" ~docs
+      Cmd.info "support-files-targets" ~docs
         ~doc:
           "Lists the names of the files that $(i,odoc support-files) outputs."
   end
 end
 
 module Occurrences = struct
-  open Or_error
-
   let dst_of_string s =
     let f = Fs.File.of_string s in
     if not (Fs.File.has_ext ".odoc-occurrences" f) then
@@ -1602,10 +1623,10 @@ module Occurrences = struct
          computed from the implementations of .odocl files found in the given \
          directories."
       in
-      Term.info "count-occurrences" ~docs ~doc
+      Cmd.info "count-occurrences" ~docs ~doc
   end
   module Aggregate = struct
-    let index dst files file_list warnings_options =
+    let index dst files file_list strip_path warnings_options =
       match (files, file_list) with
       | [], [] ->
           Error
@@ -1614,7 +1635,8 @@ module Occurrences = struct
                 to odoc aggregate-occurrences")
       | _ ->
           dst_of_string dst >>= fun dst ->
-          Occurrences.aggregate ~dst ~warnings_options files file_list
+          Occurrences.aggregate ~dst ~warnings_options ~strip_path files
+            file_list
 
     let cmd =
       let dst =
@@ -1637,20 +1659,24 @@ module Occurrences = struct
         let doc = "file created with count-occurrences" in
         Arg.(value & pos_all convert_fpath [] & info ~doc ~docv:"FILE" [])
       in
+      let strip_path =
+        let doc = "Strip package/version information from paths" in
+        Arg.(value & flag & info ~doc [ "strip-path" ])
+      in
       Term.(
         const handle_error
-        $ (const index $ dst $ inputs $ inputs_in_file $ warnings_options))
+        $ (const index $ dst $ inputs $ inputs_in_file $ strip_path
+         $ warnings_options))
 
     let info ~docs =
       let doc = "Aggregate hashtables created with odoc count-occurrences." in
-      Term.info "aggregate-occurrences" ~docs ~doc
+      Cmd.info "aggregate-occurrences" ~docs ~doc
   end
 end
 
 module Odoc_error = struct
   let errors input =
     let open Odoc_odoc in
-    let open Or_error in
     let input = Fs.File.of_string input in
     Odoc_file.load input >>= fun unit ->
     Odoc_model.Error.print_errors unit.warnings;
@@ -1663,7 +1689,7 @@ module Odoc_error = struct
   let cmd = Term.(const handle_error $ (const errors $ input))
 
   let info ~docs =
-    Term.info "errors" ~docs
+    Cmd.info "errors" ~docs
       ~doc:"Print errors that occurred while compiling or linking."
 end
 
@@ -1675,7 +1701,7 @@ module Classify = struct
   let cmd = Term.(const handle_error $ (const Classify.classify $ libdirs))
 
   let info ~docs =
-    Term.info "classify" ~docs
+    Cmd.info "classify" ~docs
       ~doc:
         "Classify the modules into libraries based on heuristics. Libraries \
          are specified by the --library option."
@@ -1714,7 +1740,7 @@ module Extract_code = struct
        $ warnings_options))
 
   let info ~docs =
-    Term.info "extract-code" ~docs
+    Cmd.info "extract-code" ~docs
       ~doc:
         "Extract code blocks from mld files in order to be able to execute them"
 end
@@ -1737,47 +1763,52 @@ let main_page_sections =
 
 let () =
   Printexc.record_backtrace true;
+  let cmd_make (term, info) = Cmd.v info term in
   let subcommands =
-    [
-      Occurrences.Count.(cmd, info ~docs:section_pipeline);
-      Occurrences.Aggregate.(cmd, info ~docs:section_pipeline);
-      Compile.(cmd, info ~docs:section_pipeline);
-      Compile_asset.(cmd, info ~docs:section_pipeline);
-      Odoc_link.(cmd, info ~docs:section_pipeline);
-      Odoc_html.generate ~docs:section_pipeline;
-      Odoc_html.generate_source ~docs:section_pipeline;
-      Odoc_html.generate_asset ~docs:section_pipeline;
-      Support_files_command.(cmd, info ~docs:section_pipeline);
-      Compile_impl.(cmd, info ~docs:section_pipeline);
-      Indexing.(cmd, info ~docs:section_pipeline);
-      Sidebar.(cmd, info ~docs:section_pipeline);
-      Odoc_manpage.generate ~docs:section_generators;
-      Odoc_latex.generate ~docs:section_generators;
-      Odoc_html_url.(cmd, info ~docs:section_support);
-      Odoc_latex_url.(cmd, info ~docs:section_support);
-      Targets.Support_files.(cmd, info ~docs:section_support);
-      Odoc_error.(cmd, info ~docs:section_support);
-      Odoc_html.targets ~docs:section_support;
-      Odoc_html.targets_source ~docs:section_support;
-      Odoc_manpage.targets ~docs:section_support;
-      Odoc_latex.targets ~docs:section_support;
-      Depends.Compile.(cmd, info ~docs:section_support);
-      Targets.Compile.(cmd, info ~docs:section_support);
-      Html_fragment.(cmd, info ~docs:section_legacy);
-      Odoc_html.process ~docs:section_legacy;
-      Odoc_manpage.process ~docs:section_legacy;
-      Odoc_latex.process ~docs:section_legacy;
-      Depends.Link.(cmd, info ~docs:section_legacy);
-      Css.(cmd, info ~docs:section_deprecated);
-      Depends.Odoc_html.(cmd, info ~docs:section_deprecated);
-      Classify.(cmd, info ~docs:section_pipeline);
-      Extract_code.(cmd, info ~docs:section_pipeline);
-    ]
+    List.map ~f:cmd_make
+    @@ [
+         Occurrences.Count.(cmd, info ~docs:section_pipeline);
+         Occurrences.Aggregate.(cmd, info ~docs:section_pipeline);
+         Compile.(cmd, info ~docs:section_pipeline);
+         Compile_asset.(cmd, info ~docs:section_pipeline);
+         Odoc_link.(cmd, info ~docs:section_pipeline);
+         Odoc_html.generate ~docs:section_pipeline;
+         Odoc_html.generate_source ~docs:section_pipeline;
+         Odoc_html.generate_asset ~docs:section_pipeline;
+         Support_files_command.(cmd, info ~docs:section_pipeline);
+         Compile_impl.(cmd, info ~docs:section_pipeline);
+         Indexing.(cmd, info ~docs:section_pipeline);
+         Sidebar.(cmd, info ~docs:section_pipeline);
+         Odoc_markdown_cmd.generate ~docs:section_pipeline;
+         Odoc_markdown_cmd.generate_source ~docs:section_pipeline;
+         Odoc_markdown_cmd.targets ~docs:section_support;
+         Odoc_manpage.generate ~docs:section_generators;
+         Odoc_latex.generate ~docs:section_generators;
+         Odoc_html_url.(cmd, info ~docs:section_support);
+         Odoc_latex_url.(cmd, info ~docs:section_support);
+         Targets.Support_files.(cmd, info ~docs:section_support);
+         Odoc_error.(cmd, info ~docs:section_support);
+         Odoc_html.targets ~docs:section_support;
+         Odoc_html.targets_source ~docs:section_support;
+         Odoc_manpage.targets ~docs:section_support;
+         Odoc_latex.targets ~docs:section_support;
+         Depends.Compile.(cmd, info ~docs:section_support);
+         Targets.Compile.(cmd, info ~docs:section_support);
+         Html_fragment.(cmd, info ~docs:section_legacy);
+         Odoc_html.process ~docs:section_legacy;
+         Odoc_manpage.process ~docs:section_legacy;
+         Odoc_latex.process ~docs:section_legacy;
+         Depends.Link.(cmd, info ~docs:section_legacy);
+         Css.(cmd, info ~docs:section_deprecated);
+         Depends.Odoc_html.(cmd, info ~docs:section_deprecated);
+         Classify.(cmd, info ~docs:section_pipeline);
+         Extract_code.(cmd, info ~docs:section_pipeline);
+       ]
   in
-  let default =
+  let main =
     let print_default () =
       let available_subcommands =
-        List.map subcommands ~f:(fun (_, info) -> Term.name info)
+        List.map subcommands ~f:(fun cmd -> Cmd.name cmd)
       in
       Printf.printf
         "Available subcommands: %s\nSee --help for more information.\n%!"
@@ -1787,11 +1818,12 @@ let () =
       (* Show sections in a defined order. *)
       List.map ~f:(fun s -> `S s) main_page_sections
     in
-    ( Term.(const print_default $ const ()),
-      Term.info ~man ~version:"3.0.0" "odoc" )
+    let default = Term.(const print_default $ const ()) in
+    let info = Cmd.info ~man ~version:"3.1.0" "odoc" in
+    Cmd.group ~default info subcommands
   in
-  match Term.eval_choice ~err:Format.err_formatter default subcommands with
-  | `Error _ ->
+  match Cmd.eval_value ~err:Format.err_formatter main with
+  | Error _ ->
       Format.pp_print_flush Format.err_formatter ();
       exit 2
   | _ -> ()
