@@ -15,8 +15,8 @@
  *)
 
 open Odoc_utils
+open ResultMonad
 open Odoc_model
-open Or_error
 
 type unit_content = Lang.Compilation_unit.t
 
@@ -29,7 +29,7 @@ type content =
 type t = { content : content; warnings : Odoc_model.Error.t list }
 
 (** Written at the top of the files. Checked when loading. *)
-let magic = "odoc-3.0.0"
+let magic = "odoc-3.1.0"
 
 (** Exceptions while saving are allowed to leak. *)
 let save_ file f =
