@@ -4,6 +4,6 @@
 
 val of_type : Odoc_model.Lang.TypeExpr.t -> string
 
-val of_doc : Odoc_model.Comment.docs -> string
+val of_doc : Odoc_model.Comment.elements -> string
 
 val of_record : Odoc_model.Lang.TypeDecl.Field.t list -> string
