@@ -6,7 +6,7 @@ module rec Resolved : sig
     [ `Module of module_ | `ModuleType of module_type | `FragmentRoot ]
 
   and module_ =
-    [ `Local of Ident.path_module
+    [ `Local of Ident.module_
     | `Gpath of Path.Resolved.Module.t
     | `Substituted of module_
     | `Subst of module_type * module_
@@ -28,31 +28,24 @@ module rec Resolved : sig
     | `OpaqueModuleType of module_type ]
 
   and type_ =
-    [ `Local of Ident.path_type
+    [ `Local of Ident.type_
     | `Gpath of Path.Resolved.Type.t
     | `Substituted of type_
     | `CanonicalType of type_ * Path.Type.t
+    | `CoreType of TypeName.t
     | `Type of parent * TypeName.t
-    | `Class of parent * ClassName.t
-    | `ClassType of parent * ClassTypeName.t ]
+    | `Class of parent * TypeName.t
+    | `ClassType of parent * TypeName.t ]
 
-  and value = [ `Value of parent * ValueName.t ]
-
-  and datatype =
-    [ `Local of Ident.path_datatype
-    | `Gpath of Path.Resolved.DataType.t
-    | `Substituted of datatype
-    | `CanonicalDataType of datatype * Path.DataType.t
-    | `Type of parent * TypeName.t ]
-
-  and constructor = [ `Constructor of datatype * ConstructorName.t ]
+  and value =
+    [ `Value of parent * ValueName.t | `Gpath of Path.Resolved.Value.t ]
 
   and class_type =
-    [ `Local of Ident.path_class_type
+    [ `Local of Ident.type_
     | `Substituted of class_type
     | `Gpath of Path.Resolved.ClassType.t
-    | `Class of parent * ClassName.t
-    | `ClassType of parent * ClassTypeName.t ]
+    | `Class of parent * TypeName.t
+    | `ClassType of parent * TypeName.t ]
 end =
   Resolved
 
@@ -60,11 +53,11 @@ and Cpath : sig
   type module_ =
     [ `Resolved of Resolved.module_
     | `Substituted of module_
-    | `Local of Ident.path_module * bool
+    | `Local of Ident.module_ * bool
     | `Identifier of Identifier.Path.Module.t * bool
-    | `Root of string
+    | `Root of ModuleName.t
     | `Forward of string
-    | `Dot of module_ * string
+    | `Dot of module_ * ModuleName.t
     | `Module of Resolved.parent * ModuleName.t (* Like dot, but typed *)
     | `Apply of module_ * module_ ]
 
@@ -73,45 +66,33 @@ and Cpath : sig
     | `Substituted of module_type
     | `Local of Ident.module_type * bool
     | `Identifier of Identifier.ModuleType.t * bool
-    | `Dot of module_ * string
+    | `DotMT of module_ * ModuleTypeName.t
     | `ModuleType of Resolved.parent * ModuleTypeName.t ]
 
   and type_ =
     [ `Resolved of Resolved.type_
     | `Substituted of type_
-    | `Local of Ident.path_type * bool
+    | `Local of Ident.type_ * bool
     | `Identifier of Odoc_model.Paths.Identifier.Path.Type.t * bool
-    | `Dot of module_ * string
+    | `DotT of module_ * TypeName.t
     | `Type of Resolved.parent * TypeName.t
-    | `Class of Resolved.parent * ClassName.t
-    | `ClassType of Resolved.parent * ClassTypeName.t ]
+    | `Class of Resolved.parent * TypeName.t
+    | `ClassType of Resolved.parent * TypeName.t ]
 
   and value =
     [ `Resolved of Resolved.value
-    | `Dot of module_ * string
-    | `Value of Resolved.parent * ValueName.t ]
-
-  and datatype =
-    [ `Resolved of Resolved.datatype
-    | `Substituted of datatype
-    | `Local of Ident.path_datatype * bool
-    | `Identifier of Odoc_model.Paths.Identifier.Path.DataType.t * bool
-    | `Dot of module_ * string
-    | `Type of Resolved.parent * TypeName.t ]
-
-  and constructor =
-    [ `Resolved of Resolved.constructor
-    | `Dot of datatype * string
-    | `Constructor of Resolved.datatype * ConstructorName.t ]
+    | `DotV of module_ * ValueName.t
+    | `Value of Resolved.parent * ValueName.t
+    | `Identifier of Identifier.Value.t * bool ]
 
   and class_type =
     [ `Resolved of Resolved.class_type
     | `Substituted of class_type
-    | `Local of Ident.path_class_type * bool
+    | `Local of Ident.type_ * bool
     | `Identifier of Odoc_model.Paths.Identifier.Path.ClassType.t * bool
-    | `Dot of module_ * string
-    | `Class of Resolved.parent * ClassName.t
-    | `ClassType of Resolved.parent * ClassTypeName.t ]
+    | `DotT of module_ * TypeName.t
+    | `Class of Resolved.parent * TypeName.t
+    | `ClassType of Resolved.parent * TypeName.t ]
 end =
   Cpath
 
@@ -145,6 +126,7 @@ and is_resolved_module_type_substituted : Resolved.module_type -> bool =
 
 and is_resolved_type_substituted : Resolved.type_ -> bool = function
   | `Local _ -> false
+  | `CoreType _ -> false
   | `Substituted _ -> true
   | `Gpath _ -> false
   | `CanonicalType (t, _) -> is_resolved_type_substituted t
@@ -172,7 +154,7 @@ let is_module_type_substituted : module_type -> bool = function
   | `Identifier _ -> false
   | `Local _ -> false
   | `Substituted _ -> true
-  | `Dot (a, _) -> is_module_substituted a
+  | `DotMT (a, _) -> is_module_substituted a
   | `ModuleType (a, _) -> is_resolved_parent_substituted a
 
 let is_type_substituted : type_ -> bool = function
@@ -180,7 +162,7 @@ let is_type_substituted : type_ -> bool = function
   | `Identifier _ -> false
   | `Local _ -> false
   | `Substituted _ -> true
-  | `Dot (a, _) -> is_module_substituted a
+  | `DotT (a, _) -> is_module_substituted a
   | `Type (a, _) | `Class (a, _) | `ClassType (a, _) ->
       is_resolved_parent_substituted a
 
@@ -189,7 +171,7 @@ let is_class_type_substituted : class_type -> bool = function
   | `Identifier _ -> false
   | `Local _ -> false
   | `Substituted _ -> true
-  | `Dot (a, _) -> is_module_substituted a
+  | `DotT (a, _) -> is_module_substituted a
   | `Class (a, _) | `ClassType (a, _) -> is_resolved_parent_substituted a
 
 let rec is_module_forward : module_ -> bool = function
@@ -240,10 +222,10 @@ and is_resolved_parent_hidden :
 and is_module_type_hidden : module_type -> bool = function
   | `Resolved r -> is_resolved_module_type_hidden r
   | `Identifier ({ iv = `ModuleType (_, t); _ }, b) ->
-      b || ModuleTypeName.is_internal t
+      b || ModuleTypeName.is_hidden t
   | `Local (_, b) -> b
   | `Substituted p -> is_module_type_hidden p
-  | `Dot (p, _) -> is_module_hidden p
+  | `DotMT (p, _) -> is_module_hidden p
   | `ModuleType (p, _) -> is_resolved_parent_hidden ~weak_canonical_test:false p
 
 and is_resolved_module_type_hidden : Resolved.module_type -> bool = function
@@ -261,18 +243,17 @@ and is_resolved_module_type_hidden : Resolved.module_type -> bool = function
 
 and is_type_hidden : type_ -> bool = function
   | `Resolved r -> is_resolved_type_hidden r
-  | `Identifier ({ iv = `Type (_, t); _ }, b) -> b || TypeName.is_internal t
-  | `Identifier ({ iv = `ClassType (_, t); _ }, b) ->
-      b || ClassTypeName.is_internal t
-  | `Identifier ({ iv = `Class (_, t); _ }, b) -> b || ClassName.is_internal t
-  | `Identifier ({ iv = `CoreType _; _ }, b) -> b
+  | `Identifier ({ iv = `Type (_, t); _ }, b) -> b || TypeName.is_hidden t
+  | `Identifier ({ iv = `ClassType (_, t); _ }, b) -> b || TypeName.is_hidden t
+  | `Identifier ({ iv = `Class (_, t); _ }, b) -> b || TypeName.is_hidden t
   | `Local (_, b) -> b
-  | `Substituted p -> is_type_hidden p
-  | `Dot (p, _) -> is_module_hidden p
+  | `Substituted p -> is_type_hidden (p :> type_)
+  | `DotT (p, _) -> is_module_hidden p
   | `Type (p, _) | `Class (p, _) | `ClassType (p, _) ->
       is_resolved_parent_hidden ~weak_canonical_test:false p
 
 and is_resolved_type_hidden : Resolved.type_ -> bool = function
+  | `CoreType n -> TypeName.is_hidden n
   | `Local _ -> false
   | `Gpath p -> Odoc_model.Paths.Path.Resolved.(is_hidden (p :> t))
   | `Substituted p -> is_resolved_type_hidden p
@@ -293,7 +274,7 @@ and is_class_type_hidden : class_type -> bool = function
   | `Identifier (_, b) -> b
   | `Local (_, b) -> b
   | `Substituted p -> is_class_type_hidden p
-  | `Dot (p, _) -> is_module_hidden p
+  | `DotT (p, _) -> is_module_hidden p
   | `Class (p, _) | `ClassType (p, _) ->
       is_resolved_parent_hidden ~weak_canonical_test:false p
 
@@ -325,13 +306,13 @@ and module_of_module_reference : Reference.Module.t -> module_ = function
          | `Dot (_, _)
          | `Module (_, _) ) as parent),
         name ) ->
-      `Dot (module_of_module_reference parent, name)
+      `Dot (module_of_module_reference parent, ModuleName.make_std name)
   | `Module
       ( (( `Resolved (`Identifier { iv = #Identifier.Module.t_pv; _ })
          | `Dot (_, _)
          | `Module (_, _) ) as parent),
         name ) ->
-      `Dot (module_of_module_reference parent, ModuleName.to_string name)
+      `Dot (module_of_module_reference parent, name)
   | _ -> failwith "Not a module reference"
 
 let rec unresolve_resolved_module_path : Resolved.module_ -> module_ = function
@@ -339,7 +320,7 @@ let rec unresolve_resolved_module_path : Resolved.module_ -> module_ = function
   | `Gpath (`Identifier x) ->
       let hidden =
         match x.iv with
-        | `Module (_, n) -> Odoc_model.Names.ModuleName.is_internal n
+        | `Module (_, n) -> Odoc_model.Names.ModuleName.is_hidden n
         | _ -> false
       in
       `Identifier (x, hidden)
@@ -349,8 +330,7 @@ let rec unresolve_resolved_module_path : Resolved.module_ -> module_ = function
   | `Substituted x -> unresolve_resolved_module_path x
   | `Subst (_, x) -> unresolve_resolved_module_path x
   | `Hidden x -> unresolve_resolved_module_path x (* should assert false here *)
-  | `Module (p, m) ->
-      `Dot (unresolve_resolved_parent_path p, ModuleName.to_string m)
+  | `Module (p, m) -> `Dot (unresolve_resolved_parent_path p, m)
   | `Canonical (m, _) -> unresolve_resolved_module_path m
   | `Apply (m, a) ->
       `Apply (unresolve_resolved_module_path m, unresolve_resolved_module_path a)
@@ -366,16 +346,14 @@ and unresolve_module_path : module_ -> module_ = function
   | `Root _ as x -> x
   | `Forward _ as x -> x
   | `Dot (p, x) -> `Dot (unresolve_module_path p, x)
-  | `Module (p, x) ->
-      `Dot (unresolve_resolved_parent_path p, ModuleName.to_string x)
+  | `Module (p, x) -> `Dot (unresolve_resolved_parent_path p, x)
   | `Apply (x, y) -> `Apply (unresolve_module_path x, unresolve_module_path y)
 
 and unresolve_resolved_module_type_path : Resolved.module_type -> module_type =
   function
   | (`Local _ | `Gpath _) as p -> `Resolved p
   | `Substituted x -> unresolve_resolved_module_type_path x
-  | `ModuleType (p, n) ->
-      `Dot (unresolve_resolved_parent_path p, ModuleTypeName.to_string n)
+  | `ModuleType (p, n) -> `DotMT (unresolve_resolved_parent_path p, n)
   | `SubstT (_, m) -> unresolve_resolved_module_type_path m
   | `AliasModuleType (_, m2) -> unresolve_resolved_module_type_path m2
   | `CanonicalModuleType (p, _) -> unresolve_resolved_module_type_path p
@@ -386,32 +364,94 @@ and unresolve_resolved_parent_path : Resolved.parent -> module_ = function
   | `FragmentRoot | `ModuleType _ -> assert false
 
 and unresolve_resolved_type_path : Resolved.type_ -> type_ = function
-  | (`Gpath _ | `Local _) as p -> `Resolved p
+  | (`Gpath _ | `Local _ | `CoreType _) as p -> `Resolved p
   | `Substituted x -> unresolve_resolved_type_path x
   | `CanonicalType (t1, _) -> unresolve_resolved_type_path t1
-  | `Type (p, n) -> `Dot (unresolve_resolved_parent_path p, TypeName.to_string n)
-  | `Class (p, n) ->
-      `Dot (unresolve_resolved_parent_path p, ClassName.to_string n)
-  | `ClassType (p, n) ->
-      `Dot (unresolve_resolved_parent_path p, ClassTypeName.to_string n)
+  | `Type (p, n) -> `DotT (unresolve_resolved_parent_path p, n)
+  | `Class (p, n) -> `DotT (unresolve_resolved_parent_path p, n)
+  | `ClassType (p, n) -> `DotT (unresolve_resolved_parent_path p, n)
 
 and unresolve_resolved_class_type_path : Resolved.class_type -> class_type =
   function
   | (`Local _ | `Gpath _) as p -> `Resolved p
   | `Substituted x -> unresolve_resolved_class_type_path x
-  | `Class (p, n) ->
-      `Dot (unresolve_resolved_parent_path p, ClassName.to_string n)
-  | `ClassType (p, n) ->
-      `Dot (unresolve_resolved_parent_path p, ClassTypeName.to_string n)
+  | `Class (p, n) -> `DotT (unresolve_resolved_parent_path p, n)
+  | `ClassType (p, n) -> `DotT (unresolve_resolved_parent_path p, n)
 
 and unresolve_module_type_path : module_type -> module_type = function
   | `Resolved m -> unresolve_resolved_module_type_path m
   | y -> y
 
 and unresolve_type_path : type_ -> type_ = function
-  | `Resolved m -> unresolve_resolved_type_path m
+  | `Resolved m -> (unresolve_resolved_type_path m :> type_)
   | y -> y
 
 and unresolve_class_type_path : class_type -> class_type = function
   | `Resolved m -> unresolve_resolved_class_type_path m
   | y -> y
+
+(*
+These are left here for later. The idea is to alter a resolved path
+such that all the identifiers in it are turned into fully-qualified
+resolved paths. This is intended to be used to turn the 'original
+path' of `module type of` expressions that was resolved in the
+original context into a path that is valid in _all_ contexts.
+
+let rec original_path_cpath : module_ -> module_ option = function
+  | `Resolved p ->
+      original_path_cpath (unresolve_resolved_module_path p)
+  | `Root name -> Some (`Root name)
+  | `Forward _ -> None
+  | `Dot (p, s) -> (
+      match original_path_cpath p with
+      | Some p -> Some (`Dot (p, s))
+      | None -> None)
+  | `Apply (p1, p2) -> (
+      match (original_path_cpath p1, original_path_cpath p2) with
+      | Some p1', Some p2' -> Some (`Apply (p1', p2'))
+      | _ -> None)
+  | `Identifier (i, _) -> (
+      match original_path_module_identifier i with
+      | Some i -> Some (`Resolved i)
+      | None -> None)
+  | `Substituted p -> original_path_cpath p
+  | `Local _ ->
+      None
+  | `Module _ ->
+      None
+
+
+and original_path_module_identifier :
+    Odoc_model.Paths.Identifier.Path.Module.t -> Resolved.module_ option =
+ fun id ->
+  match id.iv with
+  | `Module (sg, name) -> (
+      match original_path_parent_identifier sg with
+      | Some sg' -> Some (`Module (sg', name))
+      | None -> None)
+  | `Root _ -> Some (`Gpath (`Identifier id))
+  | _ ->
+      None
+
+and original_path_parent_identifier :
+    Odoc_model.Paths.Identifier.Signature.t -> Resolved.parent option =
+ fun id ->
+  match id with
+  | { iv = `Module _ | `Root _ | `Parameter _ | `Result _; _ } as mid -> (
+      match original_path_module_identifier mid with
+      | Some m -> Some (`Module m)
+      | None -> None)
+  | { iv = `ModuleType _; _ } as mtid -> (
+      match original_path_module_type_identifier mtid with
+      | Some m -> Some (`ModuleType m)
+      | None -> None)
+
+and original_path_module_type_identifier :
+    Odoc_model.Paths.Identifier.ModuleType.t -> Resolved.module_type option =
+ fun id ->
+  match id.iv with
+  | `ModuleType (sg, name) -> (
+      match original_path_parent_identifier sg with
+      | Some sg' -> Some (`ModuleType (sg', name))
+      | None -> None)
+*)
