@@ -7,9 +7,10 @@ type module_type = [ `FModuleType of ModuleTypeName.t * ModuleType.t ]
 
 type datatype = [ `FType of TypeName.t * TypeDecl.t ]
 
+type core_type = [ `CoreType of TypeName.t ]
+
 type class_ =
-  [ `FClass of ClassName.t * Class.t
-  | `FClassType of ClassTypeName.t * ClassType.t ]
+  [ `FClass of TypeName.t * Class.t | `FClassType of TypeName.t * ClassType.t ]
 
 type value = [ `FValue of ValueName.t * Value.t ]
 
@@ -32,9 +33,12 @@ type label_parent = [ signature | type_ ]
 
 type constructor = [ `FConstructor of TypeDecl.Constructor.t ]
 
+type polymorphic_constructor =
+  [ `FPoly of TypeExpr.Polymorphic_variant.Constructor.t ]
+
 type field = [ `FField of TypeDecl.Field.t ]
 
-type any_in_type = [ constructor | field ]
+type any_in_type = [ constructor | field | polymorphic_constructor ]
 
 type any_in_type_in_sig =
   [ `In_type of Odoc_model.Names.TypeName.t * TypeDecl.t * any_in_type ]
@@ -91,49 +95,48 @@ let rec disambiguate = function
 
 let module_in_sig sg name =
   find_in_sig sg (function
-    | Signature.Module (id, _, m) when N.module_ id = name ->
+    | Signature.Module (id, _, m)
+      when ModuleName.equal_modulo_shadowing (N.typed_module id) name ->
         Some (`FModule (N.typed_module id, Delayed.get m))
     | _ -> None)
 
 let module_type_in_sig sg name =
   find_in_sig sg (function
-    | Signature.ModuleType (id, mt) when N.module_type id = name ->
+    | Signature.ModuleType (id, mt)
+      when ModuleTypeName.equal_modulo_shadowing (N.typed_module_type id) name
+      ->
         Some (`FModuleType (N.typed_module_type id, Delayed.get mt))
     | _ -> None)
 
 let type_in_sig sg name =
   find_in_sig sg (function
-    | Signature.Type (id, _, m) when N.type_ id = name ->
-        Some (`FType (N.type' id, Delayed.get m))
-    | Class (id, _, c) when N.class_ id = name ->
-        Some (`FClass (N.class' id, c))
-    | ClassType (id, _, c) when N.class_type id = name ->
-        Some (`FClassType (N.class_type' id, c))
-    | _ -> None)
-
-let datatype_in_sig sg name =
-  find_in_sig sg (function
-    | Signature.Type (id, _, m) when N.type_ id = name ->
-        Some (`FType (N.type' id, Delayed.get m))
+    | Signature.Type (id, _, m)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FType (N.typed_type id, Delayed.get m))
+    | Class (id, _, c)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FClass (N.typed_type id, c))
+    | ClassType (id, _, c)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FClassType (N.typed_type id, c))
     | _ -> None)
 
 type removed_type =
   [ `FType_removed of TypeName.t * TypeExpr.t * TypeDecl.Equation.t ]
 
-type careful_module = [ module_ | `FModule_removed of Cpath.Resolved.module_ ]
+type careful_module = [ module_ | `FModule_removed of Cpath.module_ ]
 
 type careful_module_type =
   [ module_type | `FModuleType_removed of ModuleType.expr ]
 
-type careful_type = [ type_ | removed_type ]
-
-type careful_datatype = [ datatype | removed_type ]
+type careful_type = [ type_ | removed_type | core_type ]
 
 type careful_class = [ class_ | removed_type ]
 
 let careful_module_in_sig sg name =
   let removed_module = function
-    | Signature.RModule (id, p) when N.module_ id = name ->
+    | Signature.RModule (id, p) when ModuleName.equal_modulo_shadowing id name
+      ->
         Some (`FModule_removed p)
     | _ -> None
   in
@@ -143,7 +146,8 @@ let careful_module_in_sig sg name =
 
 let careful_module_type_in_sig sg name =
   let removed_module_type = function
-    | Signature.RModuleType (id, p) when N.module_type id = name ->
+    | Signature.RModuleType (id, p)
+      when ModuleTypeName.equal_modulo_shadowing id name ->
         Some (`FModuleType_removed p)
     | _ -> None
   in
@@ -153,8 +157,8 @@ let careful_module_type_in_sig sg name =
 
 let removed_type_in_sig sg name =
   let removed_type = function
-    | Signature.RType (id, p, eq) when N.type_ id = name ->
-        Some (`FType_removed (N.type' id, p, eq))
+    | Signature.RType (id, p, eq) when id = name ->
+        Some (`FType_removed (id, p, eq))
     | _ -> None
   in
   find_map removed_type sg.Signature.removed
@@ -164,17 +168,21 @@ let careful_type_in_sig sg name =
   | Some _ as x -> x
   | None -> removed_type_in_sig sg name
 
-let careful_datatype_in_sig sg name =
-  match datatype_in_sig sg name with
-  | Some _ as x -> x
-  | None -> removed_type_in_sig sg name
+let datatype_in_sig sg name =
+  find_in_sig sg (function
+    | Signature.Type (id, _, t)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FType (N.typed_type id, Component.Delayed.get t))
+    | _ -> None)
 
 let class_in_sig sg name =
   filter_in_sig sg (function
-    | Signature.Class (id, _, c) when N.class_ id = name ->
-        Some (`FClass (N.class' id, c))
-    | Signature.ClassType (id, _, c) when N.class_type id = name ->
-        Some (`FClassType (N.class_type' id, c))
+    | Signature.Class (id, _, c)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FClass (N.typed_type id, c))
+    | Signature.ClassType (id, _, c)
+      when TypeName.equal_modulo_shadowing (N.typed_type id) name ->
+        Some (`FClassType (N.typed_type id, c))
     | _ -> None)
 
 let class_in_sig_unambiguous sg name = disambiguate (class_in_sig sg name)
@@ -184,18 +192,6 @@ let careful_class_in_sig sg name =
   | Some _ as x -> x
   | None -> removed_type_in_sig sg name
 
-let constructor_in_type (typ : TypeDecl.t) name =
-  let rec find_cons = function
-    | ({ TypeDecl.Constructor.name = name'; _ } as cons) :: _ when name' = name
-      ->
-        Some (`FConstructor cons)
-    | _ :: tl -> find_cons tl
-    | [] -> None
-  in
-  match typ.representation with
-  | Some (Variant cons) -> find_cons cons
-  | Some (Record _) | Some Extensible | None -> None
-
 let any_in_type (typ : TypeDecl.t) name =
   let rec find_cons = function
     | ({ TypeDecl.Constructor.name = name'; _ } as cons) :: _ when name' = name
@@ -210,10 +206,23 @@ let any_in_type (typ : TypeDecl.t) name =
     | _ :: tl -> find_field tl
     | [] -> None
   in
+  let rec find_poly = function
+    | TypeExpr.Polymorphic_variant.Constructor
+        ({ TypeExpr.Polymorphic_variant.Constructor.name = name'; _ } as cons)
+      :: _
+      when name' = name || name = "`" ^ name' ->
+        Some (`FPoly cons)
+    | _ :: tl -> find_poly tl
+    | [] -> None
+  in
   match typ.representation with
   | Some (Variant cons) -> find_cons cons
   | Some (Record fields) -> find_field fields
-  | Some Extensible | None -> None
+  | Some Extensible -> None
+  | None -> (
+      match typ.equation.manifest with
+      | Some (Polymorphic_variant pv) -> find_poly pv.elements
+      | Some _ | None -> None)
 
 let any_in_typext (typext : Extension.t) name =
   let rec inner = function
@@ -230,7 +239,7 @@ let any_in_comment d name =
     match xs with
     | elt :: rest -> (
         match elt.Odoc_model.Location_.value with
-        | `Heading lbl when Ident.Name.label lbl.Label.label = name ->
+        | `Heading lbl when Ident.Name.typed_label lbl.Label.label = name ->
             Some (`FLabel lbl)
         | _ -> inner rest)
     | [] -> None
@@ -246,23 +255,23 @@ let any_in_sig sg name =
     | ModuleType (id, mt) when N.module_type id = name ->
         Some (`FModuleType (N.typed_module_type id, Delayed.get mt))
     | Type (id, _, t) when N.type_ id = name ->
-        Some (`FType (N.type' id, Delayed.get t))
+        Some (`FType (N.typed_type id, Delayed.get t))
     | TypeSubstitution (id, ts) when N.type_ id = name -> Some (`FType_subst ts)
     | Exception (id, exc) when N.exception_ id = name ->
         Some (`FExn (N.typed_exception id, exc))
     | Value (id, v) when N.value id = name ->
         Some (`FValue (N.typed_value id, Delayed.get v))
-    | Class (id, _, c) when N.class_ id = name ->
-        Some (`FClass (N.class' id, c))
-    | ClassType (id, _, ct) when N.class_type id = name ->
-        Some (`FClassType (N.class_type' id, ct))
+    | Class (id, _, c) when N.type_ id = name ->
+        Some (`FClass (N.typed_type id, c))
+    | ClassType (id, _, ct) when N.type_ id = name ->
+        Some (`FClassType (N.typed_type id, ct))
     | Type (id, _, t) -> (
         let typ = Delayed.get t in
         match any_in_type typ name with
-        | Some r -> Some (`In_type (N.type' id, typ, r))
+        | Some r -> Some (`In_type (N.typed_type id, typ, r))
         | None -> None)
     | TypExt typext -> any_in_typext typext name
-    | Comment (`Docs d) -> any_in_comment d name
+    | Comment (`Docs d) -> any_in_comment d.elements (LabelName.make_std name)
     | _ -> None)
 
 let signature_in_sig sg name =
@@ -275,32 +284,39 @@ let signature_in_sig sg name =
 
 let module_type_in_sig sg name =
   find_in_sig sg (function
-    | Signature.ModuleType (id, m) when N.module_type id = name ->
+    | Signature.ModuleType (id, m)
+      when ModuleTypeName.equal_modulo_shadowing (N.typed_module_type id) name
+      ->
         Some (`FModuleType (N.typed_module_type id, Delayed.get m))
     | _ -> None)
 
 let value_in_sig sg name =
-  filter_in_sig sg (function
-    | Signature.Value (id, m) when N.value id = name ->
+  find_in_sig sg (function
+    | Signature.Value (id, m)
+      when ValueName.equal_modulo_shadowing (N.typed_value id) name
+           || ValueName.to_string (N.typed_value id)
+              = "(" ^ ValueName.to_string name ^ ")" ->
+        (* For operator, the value will have name [(<op>)]. We match that even
+           with name [<op>]. *)
         Some (`FValue (N.typed_value id, Delayed.get m))
     | _ -> None)
 
-let value_in_sig_unambiguous sg name = disambiguate (value_in_sig sg name)
-
 let label_in_sig sg name =
   filter_in_sig sg (function
-    | Signature.Comment (`Docs d) -> any_in_comment d name
+    | Signature.Comment (`Docs d) -> any_in_comment d.elements name
     | _ -> None)
 
 let exception_in_sig sg name =
   find_in_sig sg (function
-    | Signature.Exception (id, e) when N.exception_ id = name ->
+    | Signature.Exception (id, e) when N.typed_exception id = name ->
         Some (`FExn (N.typed_exception id, e))
     | _ -> None)
 
 let extension_in_sig sg name =
   let rec inner t = function
-    | ec :: _ when ec.Extension.Constructor.name = name -> Some (`FExt (t, ec))
+    | ec :: _ when ec.Extension.Constructor.name = ExtensionName.to_string name
+      ->
+        Some (`FExt (t, ec))
     | _ :: tl -> inner t tl
     | [] -> None
   in
@@ -315,11 +331,11 @@ let label_parent_in_sig sg name =
     | ModuleType (id, mt) when N.module_type id = name ->
         Some (`FModuleType (N.typed_module_type id, Component.Delayed.get mt))
     | Type (id, _, t) when N.type_ id = name ->
-        Some (`FType (N.type' id, Component.Delayed.get t))
-    | Class (id, _, c) when N.class_ id = name ->
-        Some (`FClass (N.class' id, c))
-    | ClassType (id, _, c) when N.class_type id = name ->
-        Some (`FClassType (N.class_type' id, c))
+        Some (`FType (N.typed_type id, Component.Delayed.get t))
+    | Class (id, _, c) when N.type_ id = name ->
+        Some (`FClass (N.typed_type id, c))
+    | ClassType (id, _, c) when N.type_ id = name ->
+        Some (`FClassType (N.typed_type id, c))
     | _ -> None)
 
 let any_in_type_in_sig sg name =
@@ -327,7 +343,7 @@ let any_in_type_in_sig sg name =
     | Signature.Type (id, _, t) -> (
         let t = Delayed.get t in
         match any_in_type t name with
-        | Some x -> Some (`In_type (N.type' id, t, x))
+        | Some x -> Some (`In_type (N.typed_type id, t, x))
         | None -> None)
     | _ -> None)
 
@@ -356,13 +372,13 @@ let any_in_class_signature cs name =
 
 let method_in_class_signature cs name =
   find_in_class_signature cs (function
-    | ClassSignature.Method (id, m) when N.method_ id = name ->
-        Some (`FMethod (N.typed_method id, m))
+    | ClassSignature.Method (id, m) when N.typed_method id = name ->
+        Some (`FMethod (name, m))
     | _ -> None)
 
 let instance_variable_in_class_signature cs name =
   find_in_class_signature cs (function
     | ClassSignature.InstanceVariable (id, iv)
-      when N.instance_variable id = name ->
-        Some (`FInstance_variable (N.typed_instance_variable id, iv))
+      when N.typed_instance_variable id = name ->
+        Some (`FInstance_variable (name, iv))
     | _ -> None)
