@@ -442,7 +442,8 @@ let warn_on_hidden_representation (id : Id.Type.t)
         || List.exists (fun t -> internal_typ_exp t) ts
     | Poly (_, t) | Alias (t, _) -> internal_typ_exp t
     | Arrow (_, t, t2) -> internal_typ_exp t || internal_typ_exp t2
-    | Tuple ts | Class (_, ts) -> List.exists (fun t -> internal_typ_exp t) ts
+    | Tuple ts -> List.exists (fun (_, t) -> internal_typ_exp t) ts
+    | Class (_, ts) -> List.exists (fun t -> internal_typ_exp t) ts
     | _ -> false
   in
 
@@ -761,7 +762,7 @@ and handle_fragments env id sg subs =
   List.fold_left
     (fun (sg_res, subs) lsub ->
       match (sg_res, lsub) with
-      | Result.Ok sg, ModuleEq (frag, decl) ->
+      | Ok sg, ModuleEq (frag, decl) ->
           let frag' =
             match frag with
             | `Resolved f ->
@@ -1103,7 +1104,11 @@ and type_expression : Env.t -> Id.Signature.t -> _ -> _ =
         ( lbl,
           type_expression env parent visited t1,
           type_expression env parent visited t2 )
-  | Tuple ts -> Tuple (List.map (type_expression env parent visited) ts)
+  | Tuple ts ->
+      Tuple
+        (List.map
+           (fun (lbl, ty) -> (lbl, type_expression env parent visited ty))
+           ts)
   | Constr (path', ts') -> (
       let path = type_path env path' in
       let ts = List.map (type_expression env parent visited) ts' in
