@@ -123,7 +123,8 @@ let rec substitute_vars vars t =
   | Alias (t, str) -> Alias (substitute_vars vars t, str)
   | Arrow (lbl, t1, t2) ->
       Arrow (lbl, substitute_vars vars t1, substitute_vars vars t2)
-  | Tuple ts -> Tuple (List.map (substitute_vars vars) ts)
+  | Tuple ts ->
+      Tuple (List.map (fun (lbl, ty) -> (lbl, substitute_vars vars ty)) ts)
   | Constr (p, ts) -> Constr (p, List.map (substitute_vars vars) ts)
   | Polymorphic_variant v ->
       Polymorphic_variant (substitute_vars_poly_variant vars v)
@@ -546,7 +547,7 @@ and type_expr s t =
   | Any -> Any
   | Alias (t, str) -> Alias (type_expr s t, str)
   | Arrow (lbl, t1, t2) -> Arrow (lbl, type_expr s t1, type_expr s t2)
-  | Tuple ts -> Tuple (List.map (type_expr s) ts)
+  | Tuple ts -> Tuple (List.map (fun (lbl, ty) -> (lbl, type_expr s ty)) ts)
   | Constr (p, ts) -> (
       match type_path s p with
       | Replaced (t, eq) ->
