@@ -15,3 +15,12 @@ module type X = B
 module type Y = A
 
 module type Z = A
+
+
+(* Test of loop *)
+
+(** @canonical Test.AB *)
+module type AA = sig type t end
+
+(** @canonical Test.AB *)
+module type AB = AA
\ No newline at end of file
